Package: iCARE
Title: Individualized Coherent Absolute Risk Estimation (iCARE)
Version: 1.39.0
Date: 2018-12-03
Author: Parichoy Pal Choudhury, Paige Maas, William Wheeler, Nilanjan Chatterjee
Description: An R package to build, validate and apply absolute risk models
Maintainer: Parichoy Pal Choudhury <Parichoy.PalChoudhury@cancer.org>
Depends: R (>= 3.3.0), plotrix, gtools, Hmisc
Suggests: RUnit, BiocGenerics
License: GPL-3 + file LICENSE
LazyData: true
biocViews: Software, StatisticalMethod, GenomeWideAssociation
NeedsCompilation: yes
Packaged: 2025-10-30 21:35:30 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/iCARE
git_branch: devel
git_last_commit: 5d81f64
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30

import(stats, utils, graphics, plotrix, gtools, Hmisc)
useDynLib(iCARE)
export(computeAbsoluteRisk, computeAbsoluteRiskSplitInterval, ModelValidation, plotModelValidation)
S3method(print, icare)
S3method(summary, icare)
S3method(plot, icare)
S3method(print, icareSplit)
S3method(summary, icareSplit)
S3method(plot, icareSplit)
S3method(print, icareValid)
S3method(plot, icareValid)

[![PyPI version](https://badge.fury.io/py/icare.svg)](https://badge.fury.io/py/icare)
[![codecov](https://codecov.io/github/Lrebaud/ICARE/branch/main/graph/badge.svg?token=W4D5C373NR)](https://codecov.io/github/Lrebaud/ICARE)

<h1>
  <img align="right" height="80" src="https://raw.githubusercontent.com/Lrebaud/ICARE/main/img/logo.png">
   <br> ICARE
</h1>

> Rebaud, L.\*, Escobar, T.\*, Khalid, F., Girum, K., Buvat, I.: 
Head and Neck Tumor and Lymph Node Segmentation and Outcome Prediction from 18F-FDG PET/CT Images: Simplicity is All You Need. 
In: Lecture Notes in Computer Science (LNCS) Challenges (under review)

This package contains the Individual Coefficient Approximation for Risk Estimation
(ICARE) survival model. For ensembling strategies, it also includes a dedicated bagging aggregator.

This model was the winning solution for the [MICCAI 2022](https://conferences.miccai.org/2022/en/) challenge: HEad and neCK TumOR ([HECKTOR](https://hecktor.grand-challenge.org/)) for the outcome prediction task from PET/CT.

## Description

The Individual Coefficient Approximation for Risk Estimation (ICARE) survival model
uses a minimal learning strategy to reduce to risk of overfitting on the often 
noisy and censored survival data.
To do so:
 * drop highly correlated features
 * for each feature:
   * evaluate feature sign using an univariate approach
   * normalize the feature
   * multiply the feature by its sign
 * the prediction is computed as the mean of all signed features

This makes the model more robust to overfitting. It also makes it
resilient to the curse of dimensionality. We hypothesize that it is 
better to have too many features than too few for this model.
This algorithm is implemented in the `IcareSurv` estimator in this
package.

To improve the performance, this model can be bagged. The package
provides `BaggedIcareSurv` estimator that does the ensembling of 
multiple `IcareSurv` estimators. 

The models make predictions that are **anti-concordants** with the target. 
For instance, if the target is the survival in days since baseline, the
prediction corresponds to the **ranking risk** of death.


## Getting Started

### Dependencies

* Python 3.6 or later
* scikit-survival

### Installing

Via PyPI:
```shell
pip install icare
```
or via GitHub
```shell
pip install git+https://github.com/Lrebaud/ICARE.git
```

### Documentation
Coming soon.

### Utilisation

The model is used as any other scikit-learn estimator:
```python
from sksurv import datasets
from sksurv.preprocessing import OneHotEncoder
from icare.metrics import harrell_cindex
from icare.survival import IcareSurv, BaggedIcareSurv


X, y = datasets.load_veterans_lung_cancer()
X = OneHotEncoder().fit_transform(X)

model = IcareSurv()
model.fit(X, y)
pred = model.predict(X)
print(pred[:5])
harrell_cindex(y, pred)
```

It can be used with all scikit-learn functions:
```python
from sklearn.model_selection import cross_val_score, ShuffleSplit

model = IcareSurv()
score = cross_val_score(model, X, y,
                        cv=ShuffleSplit(n_splits=20, test_size=.25),
                        n_jobs=-1,
                        scoring=harrell_cindex_scorer).mean()

```

If you are working with a censored target, you need to create a
structured array containing both the time and the censoring:

```python
from sksurv.util import Surv
y = Surv.from_arrays(event=np.array(event_happened).astype('bool'),
                     time=time)
```


## Author

Louis Rebaud: [louis.rebaud@gmail.com](mailto:louis.rebaud@gmail.com)

## Version History

* 0.0.1
    * Initial Release

## License

This project is licensed under the Apache License 2.0 - see the LICENSE.md file for details

## Acknowledgements

This package was created as a part of the HEad and neCK TumOR segmentation and outcome prediction in PET/CT images challenge, 3<sup>rd</sup> edition (HECKTOR 2022) and presented by Louis Rebaud and Thibault Escobar at the 25<sup>th</sup> international conference on Medical Image Computing and Computer-Assisted Intervention (MICCAI) congress in Singapore.

## Citation

If you use this package for your research, please cite the following [paper](https://dx.doi.org/10.13140/RG.2.2.30709.04328) (preprint):

```blockquote
Rebaud, L.*, Escobar, T.*, Khalid, F., Girum, K., Buvat, I.: Head and Neck Tumor and Lymph Node Segmentation and Outcome Prediction from 18F-FDG PET/CT Images: Simplicity is All You Need. In: Lecture Notes in Computer Science (LNCS) Challenges (under review)
```

call_c1 <- function(final, lps, ref_full_LP, miss, ref_risks, refmat, betavec, zmat, pop.weights, ncuts = 100, debug = 0) {


  DMISS            <- 999999999.9
  DMISS_TEST       <- 999999999.0
  zmat             <- zmat[, miss, drop = FALSE]
  temp             <- !is.finite(zmat)
  zmat[temp]       <- DMISS
  temp             <- !is.finite(ref_risks)
  ref_risks[temp]  <- DMISS
  temp             <- !is.finite(refmat)
  refmat[temp]     <- DMISS
  n_beta           <- length(betavec)
  probs            <- seq(0, 1, 1/ncuts)
  n_probs          <- length(probs)
  nr_z             <- ncol(zmat)
  nc_z             <- nrow(zmat)
  nr_ref           <- ncol(refmat)
  nc_ref           <- nrow(refmat)
  retvec           <- rep(DMISS, length(miss))
  retlps           <- rep(DMISS, length(miss))
  retflag          <- 1
  dim(ref_full_LP) <- NULL

  temp <- .C("ref_risk1", as.numeric(ref_risks), as.numeric(betavec), as.numeric(t(zmat)),
       as.numeric(t(refmat)), as.integer(n_beta), as.integer(nr_z), as.integer(nc_z),
       as.integer(nr_ref), as.integer(nc_ref), as.numeric(probs), as.integer(n_probs),
       as.integer(debug), as.numeric(pop.weights), as.double(ref_full_LP),
       retvec = as.numeric(retvec), retflag = as.integer(retflag), retlps=as.numeric(retlps))

  retflag <- temp$retflag
  if (retflag) stop("ERROR in c code")
  retvec  <- temp$retvec
  retlps  <- temp$retlps
  temp    <- retvec > DMISS_TEST
  if (any(temp)) retvec[temp] <- NA
  temp    <- retlps > DMISS_TEST
  if (any(temp)) retlps[temp] <- NA

  final[miss] <- retvec
  lps[miss]   <- retlps

  list(final=final, lps=lps)

} # END: call_c1

call_c2 <- function(final, lps, ref_full_LP, miss, refmat, betavec, zmat, age_new, age_int, popSubFromLP,
                    lambda_0, compRates0, pop.weights, ncuts = 100, debug = 0) {

  DMISS            <- 999999999.9
  DMISS_TEST       <- 999999999.0
  zmat             <- zmat[, miss, drop = FALSE]
  temp             <- !is.finite(zmat)
  zmat[temp]       <- DMISS
  temp             <- !is.finite(refmat)
  refmat[temp]     <- DMISS
  n_beta           <- length(betavec)
  probs            <- seq(0, 1, 1/ncuts)
  n_probs          <- length(probs)
  nr_z             <- ncol(zmat)
  nc_z             <- nrow(zmat)
  nr_ref           <- ncol(refmat)
  nc_ref           <- nrow(refmat)
  retvec           <- rep(DMISS, length(miss))
  retlps           <- rep(DMISS, length(miss))
  retflag          <- 1
  dim(ref_full_LP) <- NULL

  age_new         <- age_new[miss]
  age_int         <- age_int[miss]

  # Get all values of age
  temp                         <- c(age_new, age_new+age_int)
  maxa                         <- max(temp)
  n_lambda                     <- maxa + 1
  lambda                       <- rep(0, n_lambda)
  lambda[lambda_0[, 1]+1]      <- lambda_0[, 2]
  compRates                    <- rep(0, n_lambda)
  compRates[compRates0[, 1]+1] <- compRates0[, 2]
  temp                         <- is.na(compRates)
  compRates[temp]              <- DMISS

  temp <- .C("ref_risk2", as.numeric(betavec), as.numeric(t(zmat)), as.numeric(t(refmat)),
            as.integer(n_beta), as.integer(nr_z), as.integer(nc_z), as.integer(nr_ref),
            as.integer(nc_ref), as.numeric(probs), as.integer(n_probs), as.integer(debug),
            as.integer(age_new), as.integer(age_int), as.integer(n_lambda), as.numeric(popSubFromLP),
            as.numeric(lambda), as.numeric(compRates), as.numeric(pop.weights), as.double(ref_full_LP),
              retvec = as.numeric(retvec), retflag = as.integer(retflag), retlps=as.numeric(retlps))

  retflag <- temp$retflag
  if (retflag) stop("ERROR in c code")
  retvec  <- temp$retvec
  retlps  <- temp$retlps
  temp    <- retvec > DMISS_TEST
  if (any(temp)) retvec[temp] <- NA
  temp    <- retlps > DMISS_TEST
  if (any(temp)) retlps[temp] <- NA

  final[miss] <- retvec
  lps[miss]   <- retlps

  list(final=final, lps=lps)

} # END: call_c2

test_call_c1 <- function() {

  final       <- rep(-9, 2)
  lps         <- rep(-9, 2)
  ref_LP      <- 0:100
  miss        <- 1
  ref_risks   <- 0:1000
  refmat      <- matrix(data=1, nrow=length(ref_risks), ncol=2)
  betavec     <- rep(1, 2)
  zmat        <- matrix(data=1, nrow=length(final), ncol=2)
  pop.weights <- rep(1, length(final)) 

  ret <- iCARE:::call_c1(final, lps, ref_LP, miss, ref_risks, refmat, betavec, zmat, pop.weights, ncuts=10) 
  x   <- ret$final[1]

  checkEqualsNumeric(x, 0.5, tolerance=1e-8)
  
}

#' Risk Stratification from Polygenic Risk Score (PRS) based on Variance Explained
#'
#' This function is used to build absolute risk models and apply them to estimate absolute risks.
#'
#' @param apply.age.start single integer or vector of integer ages for the start of the interval over which to compute absolute risk.
#' @param apply.age.interval.length single integer or vector of integer years over which absolute risk should be computed.
#' @param model.disease.incidence.rates two column matrix [ integer ages, incidence rates] or three column matrix [start age, end age, rate] with incidence rate of disease. Must fully cover age interval for estimation.
#' @param use.c.code binary indicator of whether to run the c program for fast computation.
#' @param model.competing.incidence.rates two column matrix [ integer ages, incidence rates] or three column matrix [start age, end age, rate] with incidence rate of competing events. Must fully cover age interval for estimation.
#' @param variance.explained proportion of variance explained by PRS
#' @param N number of simulated PRS values
#' @param predict.stand.vals standardized values for which to make a risk prediction
#' @return This function returns risks based on simulated PRS based on SNPs with given \code{variance.explained}.
#' @export
#' @examples
#' print("Hello world")
PRS_research <- function(apply.age.start, apply.age.interval.length, model.disease.incidence.rates, use.c.code = 1,
                         model.competing.incidence.rates = NULL, variance.explained, N = 10000,
                         predict.stand.vals = NULL){

  if(length(apply.age.start)!=1 || length(apply.age.interval.length)!=1){
    return_and_print("ERROR:  You may only choose one value for apply.age.start and one value for apply.age.interval.length.")
    stop()
  }
  apply.age.start      <- rep(apply.age.start, N)
  apply.age.interval.length <- rep(apply.age.interval.length, N)
  lambda       <- model.disease.incidence.rates
  if(is.null(model.competing.incidence.rates)){  model.competing.incidence.rates= cbind(lambda[,1], rep(0, length(lambda[,1])))   }

  if(ncol(lambda)!=2){ return_and_print("ERROR: model.disease.incidence.rates should have 2 columns: [Ages,Rates]"); stop() }

  if(ncol(model.competing.incidence.rates)!=2){ return_and_print("ERROR: model.competing.incidence.rates should have 2 columns: [Ages,Rates]"); stop() }

  if(sum(lambda[,1]%%1)!=0){ return_and_print("ERROR: The first column of model.disease.incidence.rates should be integer ages."); stop()  }

  if(sum(model.competing.incidence.rates[,1]%%1)!=0){ return_and_print("ERROR: The first column of model.competing.incidence.rates should be integer ages."); stop()  }
  if( prod(is.element(seq(range(apply.age.start)[1], range(apply.age.start+apply.age.interval.length)[2]), lambda[,1])) == 0){
    return_and_print("ERROR: The 'model.disease.incidence.rates' input must have age-specific rates for each integer age covered by the prediction intervals defined by 'apply.age.start' and 'apply.age.interval.length.'  You must make these inputs consistent with one another to proceed.")
    stop()
  }
  if( prod(is.element(seq(range(apply.age.start)[1], range(apply.age.start+apply.age.interval.length)[2]), model.competing.incidence.rates[,1])) == 0){
    return_and_print("ERROR: The 'model.competing.incidence.rates' input must have age-specific rates for each integer age covered by the prediction intervals defined by 'apply.age.start' and 'apply.age.interval.length.'  You must make these inputs consistent with one another to proceed.")
    stop()
  }

  PRS.         <- rnorm(N)
  log.OR       <- variance.explained^0.5
  pop.dist.mat <- cbind(PRS.)
  beta_est     <- as.matrix(nrow = length(c( log.OR)), ncol = 1, c( log.OR) )
  Z_new        <- t(pop.dist.mat)
  pop.weights  <- rep(1/nrow(pop.dist.mat), length(pop.dist.mat))

  approx_expectation_rr = weighted.mean(  exp( pop.dist.mat%*% beta_est), w = pop.weights ) ## all equal weights
  calc = precise_lambda0(lambda, approx_expectation_rr, beta_est, pop.dist.mat, pop.weights)
  lambda_0 = calc[[1]]
  precise_expectation_rr = calc[[2]]
  pop.dist.mat = t(pop.dist.mat )

  ###### Compute A_j  ##
  final_risks <- comp_Aj(Z_new, apply.age.start, apply.age.interval.length, lambda_0, beta_est, model.competing.incidence.rates)

  if(is.null(predict.stand.vals)==0){

    if(is.vector(predict.stand.vals)==0){
      return_and_print("ERROR: predict.stand.vals must be a vector")
      stop()
    }else{
      temp = t(cbind(predict.stand.vals))
      specific_risks <- comp_Aj(temp, apply.age.start[1:length(predict.stand.vals)], 
          apply.age.interval.length[1:length (predict.stand.vals)], lambda_0, beta_est, model.competing.incidence.rates)
    }
  }
  results      <- list()
  results$risk <- final_risks
  if(is.null(predict.stand.vals)==0){
    results$predictions.for.stand.vals <- specific_risks
  }
  results
}

package_results <- function(final_risks, Z_new, covs_in_model, handle.snps, apply.age.start, apply.age.interval.length, apply.cov.profile ,
                            model.log.RR, beta_est, apply.snp.profile, snps.names, return.lp, lps ){
  result <-list()
  result$risk           <- cbind(as.vector(final_risks))
  colnames(result$risk) <- c("Risk_Estimate")

  if(covs_in_model){
    if(handle.snps==0){
      info = cbind(as.vector(apply.age.start), as.vector(apply.age.start + apply.age.interval.length), as.vector(final_risks), apply.cov.profile )
      colnames(info) = c("Int_Start", "Int_End", "Risk_Estimate", colnames(apply.cov.profile ))
      beta.names = rownames(model.log.RR)
    }else{
      info = cbind(as.vector(apply.age.start), as.vector(apply.age.start + apply.age.interval.length), as.vector(final_risks), cbind( apply.snp.profile, apply.cov.profile ))
      colnames(info) = c("Int_Start", "Int_End", "Risk_Estimate", snps.names, colnames(apply.cov.profile ))
      beta.names = c( snps.names, rownames(model.log.RR) )
    }
  }else{
    info = cbind(as.vector(apply.age.start), as.vector(apply.age.start + apply.age.interval.length), as.vector(final_risks), apply.snp.profile)
    colnames(info) = c("Int_Start", "Int_End", "Risk_Estimate", snps.names)
    beta.names =  snps.names
  }
  result$details = info
  beta.used = beta_est
  rownames(beta.used) <- beta.names
  colnames(beta.used) <- "log OR used"
  result$beta.used    <- beta.used

  if(return.lp==TRUE){
    result$lps <- lps
  }
  result
}

# S3 methods
summary.icare <- function(object, ...) {

  ret <- summary(object$risk, ...)
  ret

} # END: summary.icare

plot.icare <- function(x, ...) {

  plot(density(x$risk, na.rm=TRUE), ...)
  NULL

} # END: plot.icare

print.icare <- function(x, ...) {

  y   <- x$details
  nc  <- ncol(y)
  if (nc > 4) {
    cols <- c(1:3, nc-1, nc)
  } else {
    cols <- 1:nc
  }
  y <- y[, cols, drop=FALSE]
  rownames(y) <- NULL
  print(y)
  invisible(x)

} # END: print.icare

summary.icareSplit <- function(object, ...) {

  ret <- summary(object$risk, ...)
  colnames(ret) <- "Risk_Estimate"
  ret

} # END: summary.icareSplit

plot.icareSplit <- function(x, ...) {

  plot(density(x$risk, na.rm=TRUE), ...)
  NULL

} # END: plot.icareSplit

print.icareSplit <- function(x, ...) {

  y   <- x$details
  cx  <- colnames(y)
  cx[1:3] <- c("Int_Start", "Cut_Age", "Int_End")
  cx[length(cx)] <- "Risk_Estimate"
  tmp <- duplicated(cx)
  if (any(tmp)) {
    cx <- cx[!tmp]
    y  <- y[,!tmp, drop=FALSE]
  }
  colnames(y) <- cx
  rownames(y) <- NULL
  print(y)
  invisible(x)

} # END: print.icareSplit

print.icareValid <- function(x, ...) {

  args     <- x$input.args
  mformula <- args[["model.formula", exact=TRUE]]
  mf       <- paste(mformula)
  
  cat("\n")
  cat(paste("Dataset: ", args$dataset, "\n", sep=""))
  cat(paste("Model Name: ", args$model.name, "\n", sep=""))
  if(is.null(mformula)){
    cat("Model formula: Likely an additive SNP-only model\n")
  } else {
    cat(paste("Model Formula: ",mf[2],mf[1],mf[3], "\n", sep=" "))
  }
  timeframe <- x$Risk_Prediction_Interval
  cat(paste("Risk Prediction Interval: ",timeframe, "\n", sep=""))
  #cat("\n")
  observed.outcome <- x$Subject_Specific_Observed_Outcome
  cat(paste("Number of study subjects: "
                                 ,length(observed.outcome), "\n", sep=""))
  #cat("\n")
  cat(paste("Number of cases: ",sum(observed.outcome), "\n", sep=""))
  #cat("\n")
  followup <- x$Adjusted_Followup
  cat(paste("Follow-up time (years) [mean,range]: [",
                      round(mean(followup),3),", (",
                      round(range(followup)[1],3),",",
                      round(range(followup)[2],3),")","]\n", sep=""))
  #cat("\n")
  study.entry.age <- args$study.data$study.entry.age
  cat(paste("Baseline age (years) [mean,range]: [", 
              round(mean(study.entry.age),3),", (",
              round(range(study.entry.age)[1],3),",",
              round(range(study.entry.age)[2],3),")","]\n", sep=""))
  cat("\n")
  cat("Absolute Risk Calibration")
  print(x$Hosmer_Lemeshow_Results)
  #cat("\n")
  cat("Relative Risk Calibration")
  print(x$RR_test_result)
  #cat("\n")
  cat("Model Discrimination\n")
  cat(paste("Estimate of AUC: ", round(x$AUC,3), "\n", sep=""))
  y <- x$CI_AUC
  cat(paste("95% CI of AUC: (", round(y[1],3), ",", round(y[2],3), ")\n", sep=""))
  cat("\n")
  cat("Overall Expected to Observed Ratio\n")
  exp_by_obs <- x$Overall_Expected_to_Observed_Ratio 
  cat(paste("Estimate: ",round(exp_by_obs,3), "\n", sep=""))
  CI_exp_by_obs <- x$CI_Overall_Expected_to_Observed_Ratio
  cat(paste("95% CI: ", "(", round(CI_exp_by_obs[1],3),
              ",", round(CI_exp_by_obs[2],3) ,")\n", sep=""))


  invisible(x)

} # END: print.icareValid

plot.icareValid <- function(x, ...) {

  args <- x$input.args
  plotModelValidation(args$study.data, x,
                                 dataset = args$dataset,
                                 model.name = args$model.name,
                                 ...) 
  
  NULL

} # END: plot.icareValid

# History: May 29, 2015 Added option use.c.code
#          Jun 02, 2015 Updated cutpoints to first round them and then 
#                       remove duplicates
#          Jun 05, 2015 Added update_cutpoints function to update the 
#                       cutpoints in case
#                       whe there are no subjects between 2 cutpoints
#          Jun 08, 2015 Reorganized to be more modular, tidied up formatting
#          Jul 02, 2015 Modified for weights

computeAbsoluteRisk <- function(model.formula = NULL, model.cov.info = NULL, 
                                  model.snp.info = NULL, model.log.RR = NULL,
                                  model.ref.dataset = NULL, 
                                  model.ref.dataset.weights = NULL,
                                  model.disease.incidence.rates,
                                  model.competing.incidence.rates = NULL,
                                  model.bin.fh.name = NA, n.imp = 5,
                                  apply.age.start, apply.age.interval.length,
                                  apply.cov.profile  = NULL,
                                  apply.snp.profile = NULL,
                                  use.c.code = 1,  return.lp = FALSE, 
                                  return.refs.risk = FALSE){

  decision      <- decide_if_SNP_only(apply.cov.profile, model.formula, 
                     model.log.RR, model.ref.dataset, model.cov.info, 
                     model.snp.info, apply.snp.profile, apply.age.start, 
                     apply.age.interval.length)
  covs_in_model <-  decision[[1]]
  if(length(decision)>1) apply.snp.profile   <-  decision[[2]]

  
  handle.snps = 1 - is.null(model.snp.info)

  if (covs_in_model) {
    temp         <- check_age_lengths(apply.age.start, apply.age.interval.length, 
                          apply.cov.profile , "apply.cov.profile ")
    apply.age.start      <- temp[[1]]
    apply.age.interval.length <- temp[[2]]

    temp         <- check_model_inputs(apply.cov.profile , model.log.RR, 
                       model.ref.dataset, model.ref.dataset.weights, 
                       model.cov.info, model.formula, n.imp)
    data_use     <- temp[[1]]
    model.log.RR   <- temp[[2]]
    model.ref.dataset.weights <- temp[[3]]

    design_covs <- make_design_matrix_covs(data_use, apply.cov.profile, 
                      model.formula)

    check_design_matrix(model.log.RR, design_covs)
    Z_new        <- t(design_covs); rm(design_covs); gc()
    beta_est     <- as.matrix(nrow = length(model.log.RR), ncol=1, model.log.RR)
    pop.dist.mat <- make_design_matrix_dist(data_use, model.ref.dataset, 
                       model.formula)
    pop.weights  <- model.ref.dataset.weights
  }

  if(handle.snps){

    check_SNP_info(model.snp.info)

    snps.names <- paste(model.snp.info[,"snp.name"])
    snps.betas <- log( model.snp.info[,"snp.odds.ratio"] )
    snps.freqs <- model.snp.info[,"snp.freq"]

    processed_info = process_SNP_info(covs_in_model, apply.snp.profile, 
                      model.bin.fh.name, apply.cov.profile , 
                      model.ref.dataset, model.snp.info)

    attenuate.fh  <-  processed_info[[1]]
    fh.pop        <-  processed_info[[2]]
    fh.cov        <-  processed_info[[3]]
    apply.snp.profile  <-  processed_info[[4]]
  }

  if (covs_in_model) {

    if(handle.snps){
      covariate_stack5 <- do.call("rbind", replicate(n.imp, pop.dist.mat, 
                                   simplify = FALSE))
      simulated_snps   <- sim_snps(snps.betas, snps.freqs, 
                              cbind( rep( fh.pop, n.imp)) )
      pop.dist.mat     <- cbind( simulated_snps, covariate_stack5 )
      pop.weights      <- rep( model.ref.dataset.weights, n.imp )

      if(attenuate.fh){
        var_prs <- sum((snps.betas^2)*2*snps.freqs*(1-snps.freqs))
        alpha   <- var_prs/2
        if(model.cov.info[[which(model.bin.fh.name==colnames(apply.cov.profile ))]]$type=="factor"){
          model.bin.fh.name = paste("as.factor(", model.bin.fh.name, ")1", sep="")
        }
        beta_est[model.bin.fh.name,1] <- beta_est[model.bin.fh.name,1] - alpha
      }
      beta_est <- as.matrix(nrow = length(c( snps.betas, beta_est)), ncol = 1, c( snps.betas, beta_est) )
      Z_new    <- rbind(t(apply.snp.profile), Z_new)
    }
  } else {
    temp         <- check_age_lengths(apply.age.start, apply.age.interval.length, 
                       apply.snp.profile, "apply.snp.profile")
    apply.age.start      <- temp[[1]]
    apply.age.interval.length <- temp[[2]]

    if(handle.snps){
      pop.dist.mat <- sim_snps(snps.betas, snps.freqs, cbind( rep( fh.pop, n.imp)) )
      pop.weights  <- rep( 1/nrow(pop.dist.mat), nrow(pop.dist.mat) )
      beta_est     <- as.matrix(nrow = length(c( snps.betas)), ncol = 1, c( snps.betas) )
      Z_new        <- t(apply.snp.profile)
    }
  }
  lambda                    <- model.disease.incidence.rates; 
  rm(model.disease.incidence.rates)
  res                       <- check_rates(model.competing.incidence.rates, lambda,  
                                         apply.age.start, apply.age.interval.length)
  lambda                    <- res[[1]]
  model.competing.incidence.rates <- res[[2]]; rm(res)
  approx_expectation_rr     <- weighted.mean(  exp( pop.dist.mat%*% beta_est), w = pop.weights, na.rm=TRUE)
  calculation               <- precise_lambda0(lambda, approx_expectation_rr, beta_est, pop.dist.mat, pop.weights)
  lambda_0                  <- calculation[[1]]
  precise_expectation_rr    <- calculation[[2]]
  pop.dist.mat              <- t(pop.dist.mat )

  ###### Compute A_j  ## only those without NA
  final_risks <- rep(NA, ncol(Z_new))
  lps <- t(Z_new)%*%beta_est
  these = which(colSums( is.na(Z_new))==0 )

  if( length(these) > 0 ){
      temp               <- subset(Z_new, select=these)
      final_risks[these] <- comp_Aj(temp, apply.age.start[these], 
             apply.age.interval.length[these], lambda_0, beta_est, 
             model.competing.incidence.rates)
  }
  miss    <- which(is.na(final_risks))
  present <- which(!is.na(final_risks))
  ref_pop <- pop.dist.mat;
  ncuts   <- 100
  t00     <- proc.time()
  
  res         <- handle_missing_data(use.c.code, apply.age.start, 
                   apply.age.interval.length, Z_new, miss, present, ncuts, final_risks, 
                   ref_pop, pop.weights, lambda_0, beta_est, 
                   model.competing.incidence.rates, lps)
  final_risks <- res[[1]]
  lps         <- res[[2]]
  
  these = which(colSums(!is.na(Z_new))==0) # nothing to match on w referent dataset
  if(length(these)>0){
    ref_risks = get_refs_risk(ref_pop, apply.age.start, apply.age.interval.length, 
          lambda_0, beta_est, model.competing.incidence.rates, handle.snps, n.imp )[[1]]
    final_risks[these] <- weighted.mean(ref_risks, w = pop.weights[1:length(ref_risks)], 
                           na.rm = TRUE) 
  }
  t11    <- proc.time() - t00;  print(t11)
  result <- package_results(final_risks, Z_new, covs_in_model, handle.snps, apply.age.start, 
              apply.age.interval.length, apply.cov.profile , model.log.RR, beta_est, 
              apply.snp.profile, snps.names, return.lp, lps)
  if(return.refs.risk) result$refs.risk <- get_refs_risk(ref_pop, apply.age.start, 
                         apply.age.interval.length, lambda_0, beta_est, 
                      model.competing.incidence.rates, handle.snps, n.imp )[[1]]
  class(result) <- "icare"

  result
}

#' Building and Applying an Absolute Risk Model:  Compute Risk over Interval Split in Two Parts
#'
#' This function is used to build an absolute risk model that incorporates different input parameters before and after a given time point. The model is then applied to estimate absolute risks.
#'
#' @param apply.age.start single integer or vector of integer ages for the start of the interval over which to compute absolute risk.
#' @param apply.age.interval.length single integer or vector of integer years over which absolute risk should be computed.
#' @param apply.cov.profile dataframe containing the covariate profiles for which absolute risk will be computed. Covariates must be in same order with same names as in \code{model.formula}.
#' @param model.formula an object of class \code{formula}: a symbolic description of the model to be fitted, e.g. Y~Parity+FamilyHistory.
#' @param model.disease.incidence.rates two column matrix [ integer ages, incidence rates] or three column matrix [start age, end age, rate] with incidence rate of disease. Must fully cover age interval for estimation.
#' @param model.log.RR vector with log odds ratios corresponding to the model params; no intercept; names must match design matrix arising from \code{model.formula} and \code{model.cov.info}; check names using function \code{check_design_matrix()}.
#' @param model.ref.dataset dataframe of risk factors for a sample of subjects representative of underlying population, no missing values. Variables must be in same order with same names as in \code{model.formula}.
#' @param model.ref.dataset.weights optional vector of sampling weights for \code{model.ref.dataset}.
#' @param model.cov.info contains information about the risk factors in the model ;  a main list containing a list for each covariate, which must have the fields:\cr
#'        \itemize{ \item \code{"name"} : a string with the covariate name, matching name in model.formula \cr
#'        \item \code{"type"} : a string that is either "continuous" or "factor".}
#'        If factor variable, then:\cr
#'        \itemize{\item \code{"levels"} : vector with strings of level names  \cr
#'        \item \code{"ref"} : optional field, string with name of referent level
#'        }
#' @param use.c.code binary indicator of whether to run the c program for fast computation.
#' @param model.competing.incidence.rates two column matrix [ integer ages, incidence rates] or three column matrix [start age, end age, rate] with incidence rate of competing events. Must fully cover age interval for estimation.
#' @param return.lp binary indicator of whether to return the linear predictor for each subject in apply.cov.profile.
#' @param apply.snp.profile data frame with observed SNP data (coded 0,1, 2, or NA). May have missing values.
#' @param model.snp.info dataframe with three columns [ rs number, odds ratio, allele frequency ]
#' @param model.bin.fh.name string name of family history variable, if in model. This must refer to a variable that only takes values 0,1, NA.
#' @param apply.snp.profile data frame with observed SNP data (coded 0,1, 2, or NA). May have missing values.
#' @param cut.time integer age for which to split computation into before and after
#' @param apply.cov.profile.2 see \code{apply.cov.profile}, to be used for estimation in ages after the cutpoint
#' @param model.formula.2 see \code{model.formula}, to be used for estimation in ages after the cutpoint
#' @param model.log.RR.2 see \code{model.log.RR}, to be used for estimation in ages after the cutpoint
#' @param model.ref.dataset.2 see \code{model.ref.dataset}, to be used for estimation in ages after the cutpoint
#' @param model.ref.dataset.weights.2 see \code{model.ref.dataset.weights}, to be used for estimation in ages after the cutpoint
#' @param model.cov.info.2 see \code{model.cov.info}, to be used for estimation in ages after the cutpoint
#' @param model.bin.fh.name.2 see \code{model.bin.fh.name}, to be used for estimation in ages after the cutpoint
#' @param n.imp integer value for number of imputations for handling missing SNPs.
#' @details Individualized Coherent Absolute Risk Estimators (iCARE) is a tool that allows researchers to quickly build models for absolute risk and apply them to estimate individuals' risk based on a set of user defined input parameters. The software gives users the flexibility to change or update models rapidly based on new risk factors or tailor models to different populations based on the specification of simply three input arguments: \itemize{ \item (1) a model for relative risk assumed to be externally derived \item (2) an age-specific disease incidence rate and \item (3) the distribution of risk factors for the population of interest.} The tool can handle missing information on risk factors for risk estimation using an approach where all estimates are derived from a single model through appropriate model averaging.
#' @return This function returns a list of results objects, including: \cr
#' \itemize{ \item \code{risk} : absolute risk estimates over the specified interval for subjects given by \code{apply.cov.profile} \cr
#' \item \code{details}: dataframe with the start of the interval, the end of the interval, the covariate profile, and the risk estimates for each individual \cr
#' \item \code{beta.used} : the log odds ratios used in the model \cr
#' \item \code{lps} : linear predictors for subjects in \code{model.cov.profile}, if requested by \code{return.lp} \cr
#' \item \code{refs.risk} : absolute risk estimates for subjects in \code{model.ref.dataset}, if requested by \code{return.refs.risk} }
#' @export
#' @examples
#' print("Hello world")
#' 
#' 
computeAbsoluteRiskSplitInterval <- function(apply.age.start, apply.age.interval.length, apply.cov.profile, model.formula, model.disease.incidence.rates,
                    model.log.RR, model.ref.dataset, model.ref.dataset.weights=NULL, model.cov.info, use.c.code=1, model.competing.incidence.rates = NULL,
                    return.lp = FALSE, apply.snp.profile = NULL, model.snp.info = NULL, model.bin.fh.name = NULL, cut.time = NULL, apply.cov.profile.2 = NULL,
                    model.formula.2 = NULL, model.log.RR.2 = NULL, model.ref.dataset.2 = NULL, model.ref.dataset.weights.2 = NULL, model.cov.info.2 = NULL, model.bin.fh.name.2 = NULL, n.imp = 5, return.refs.risk=FALSE){

  ## if any of the second arguments are specified, then two two version
  if( is.null( cut.time )*is.null( apply.cov.profile.2 )*is.null( model.formula.2 )*is.null( model.log.RR.2 )*is.null( model.ref.dataset.2 )*is.null( model.cov.info.2 ) ==0 ){

    if(is.null( cut.time )){
      return_and_print(paste("ERROR: If you wish to use different model inputs over parts of the age interval, must specify cut.time.", sep="") )
      stop()
    }
    if( sum( (cut.time < apply.age.start) + (cut.time > apply.age.start + apply.age.interval.length) ) >0 ){
      cat("\nNote: You provided cut.times outside the interval defined by apply.age.start and apply.age.interval.length.\n")
      cat("Will compute risk using either inputs1 or inputs2 only, depending on whether the interval is below or above the cutpoint.\n")
    }
    ## for the rest, if not specified use originals
    if(is.null( apply.cov.profile.2 )){
      apply.cov.profile.2 = apply.cov.profile
    }
    if(is.null( model.formula.2 )){
      model.formula.2 = model.formula
    }

    if(is.null( model.log.RR.2 )){
      model.log.RR.2 = model.log.RR
    }
    if(is.null( model.ref.dataset.2 )){
      model.ref.dataset.2 = model.ref.dataset
    }
    if(is.null( model.cov.info.2 )){
      model.cov.info.2 = model.cov.info
    }
    if(is.null( model.bin.fh.name.2 )){
      model.bin.fh.name.2 = model.bin.fh.name
    }

    apply.age.start.1 = apply.age.start
    apply.age.interval.length.1 = (cut.time - apply.age.start)*( (cut.time - apply.age.start) < apply.age.interval.length) +   (apply.age.interval.length)*( 1 - ( (cut.time - apply.age.start) < apply.age.interval.length))
    apply.age.interval.length.1[which(apply.age.interval.length.1<0)]=0

    apply.age.start.2 = apply.age.start.1 + apply.age.interval.length.1
    apply.age.interval.length.2 = apply.age.start + apply.age.interval.length - apply.age.start.2
    
    result1 = computeAbsoluteRisk(apply.age.start=apply.age.start.1, apply.age.interval.length=apply.age.interval.length.1, apply.cov.profile=apply.cov.profile  , model.formula=model.formula,   model.disease.incidence.rates=model.disease.incidence.rates, model.log.RR=model.log.RR,    model.ref.dataset=model.ref.dataset,   model.ref.dataset.weights=model.ref.dataset.weights,  model.cov.info=model.cov.info,   use.c.code=use.c.code, model.competing.incidence.rates=model.competing.incidence.rates, return.lp=return.lp, apply.snp.profile = apply.snp.profile, model.snp.info = model.snp.info,  model.bin.fh.name = model.bin.fh.name,  n.imp = n.imp, return.refs.risk=return.refs.risk )
    result2 = computeAbsoluteRisk(apply.age.start=apply.age.start.2, apply.age.interval.length=apply.age.interval.length.2, apply.cov.profile=apply.cov.profile.2, model.formula=model.formula.2, model.disease.incidence.rates=model.disease.incidence.rates, model.log.RR=model.log.RR.2, model.ref.dataset=model.ref.dataset.2, model.ref.dataset.weights=model.ref.dataset.weights.2, model.cov.info=model.cov.info.2, use.c.code=use.c.code, model.competing.incidence.rates=model.competing.incidence.rates, return.lp=return.lp, apply.snp.profile = apply.snp.profile, model.snp.info = model.snp.info,  model.bin.fh.name = model.bin.fh.name.2, n.imp = n.imp, return.refs.risk=return.refs.risk )

    result = combine_risks_two_intervals(result1$details[,"Int_Start"], result1$details[,"Int_End"] - result1$details[,"Int_Start"], result1$details[,"Risk_Estimate"], result2$details[,"Int_Start"], result2$details[,"Int_End"] - result2$details[,"Int_Start"], result2$details[,"Risk_Estimate"])
    result$details = cbind(apply.age.start.1, apply.age.start.2, apply.age.start.2 + apply.age.interval.length.2, apply.cov.profile , apply.cov.profile.2, result$risk)
    if(return.lp){
      result$lps.1 = result1$lps
      result$lps.2 = result2$lps
    }
    if(return.refs.risk){
      result$refs.risk.1 = result1$refs.risk
      result$refs.risk.2 = result2$refs.risk}
  }
  else{  # just run standard function
    result = computeAbsoluteRisk(apply.age.start=apply.age.start, apply.age.interval.length=apply.age.interval.length, apply.cov.profile=apply.cov.profile , model.formula=model.formula, model.disease.incidence.rates=model.disease.incidence.rates, model.log.RR=model.log.RR, model.ref.dataset=model.ref.dataset, model.ref.dataset.weights=model.ref.dataset.weights, model.cov.info=model.cov.info, use.c.code=use.c.code, model.competing.incidence.rates=model.competing.incidence.rates, return.lp=return.lp, apply.snp.profile = apply.snp.profile, model.snp.info = model.snp.info, model.bin.fh.name = model.bin.fh.name, n.imp = n.imp)
  }
  class(result) <- "icareSplit"

  result
}

test_risk_split <- function() {

  risks <- c(0.08589966, 0.07212156, 0.14428262)

  data(bc_data, package="iCARE")
  set.seed(1)
  
  results <- computeAbsoluteRiskSplitInterval(model.formula=bc_model_formula, 
                                         cut.time = 50,
                                         model.cov.info       = bc_model_cov_info,
                                         model.snp.info       = bc_72_snps,
                                         model.log.RR         = bc_model_log_or,
                                         model.log.RR.2       = bc_model_log_or_post_50,
                                         model.ref.dataset    = ref_cov_dat,
                                         model.ref.dataset.2  = ref_cov_dat_post_50,
                                         model.disease.incidence.rates   = bc_inc,
                                         model.competing.incidence.rates = mort_inc, 
                                         model.bin.fh.name = "famhist",
                                         apply.age.start    = 30, 
                                         apply.age.interval.length = 40,
                                         apply.cov.profile  = new_cov_prof,
                                         apply.snp.profile  = new_snp_prof, 
                                         return.refs.risk   = TRUE)
  ret <- as.numeric(results$risk)

  checkEqualsNumeric(ret, risks, tolerance=1.0e-4)
  
}

test_risk <- function() {

  risks <- c(0.10250188, 0.09017017, 0.16893543)

  data(bc_data, package="iCARE")
  set.seed(1)
  results <- computeAbsoluteRisk(model.formula     = bc_model_formula, 
                                         model.cov.info    = bc_model_cov_info,
                                         model.snp.info    = bc_72_snps,
                                         model.log.RR      = bc_model_log_or,
                                         model.ref.dataset = ref_cov_dat,
                                         model.disease.incidence.rates   = bc_inc,
                                         model.competing.incidence.rates = mort_inc, 
                                         model.bin.fh.name = "famhist",
                                         apply.age.start    = 50, 
                                         apply.age.interval.length = 30,
                                         apply.cov.profile  = new_cov_prof,
                                         apply.snp.profile  = new_snp_prof, 
                                         return.refs.risk   = TRUE)
  ret <- as.numeric(results$risk)
  
  
  checkEqualsNumeric(ret, risks, tolerance=1.0e-4)
  
}

### R code from vignette source 'vignette_model_validation.Rnw'

###################################################
### code chunk number 1: start
###################################################
library(iCARE)


###################################################
### code chunk number 2: load-data
###################################################
data("bc_data", package="iCARE")
set.seed(50)


###################################################
### code chunk number 3: model-1a
###################################################
res_snps_miss = computeAbsoluteRisk(model.snp.info = bc_72_snps,
                       model.disease.incidence.rates = bc_inc,
                       model.competing.incidence.rates = mort_inc,
                       apply.age.start = 50, apply.age.interval.length = 30,
                       return.refs.risk = TRUE)


###################################################
### code chunk number 4: summary-1a
###################################################
summary(res_snps_miss$refs.risk)


###################################################
### code chunk number 5: model 1b
###################################################
res_snps_dat = computeAbsoluteRisk(model.snp.info = bc_72_snps,
                     model.disease.incidence.rates = bc_inc,
                     model.competing.incidence.rates = mort_inc,
                     apply.age.start = 50, apply.age.interval.length = 30,
                     apply.snp.profile = new_snp_prof,
                     return.refs.risk = TRUE)
names(res_snps_dat)


###################################################
### code chunk number 6: plot-1b
###################################################
plot(density(res_snps_dat$refs.risk),
           xlim = c(0.04,0.18), xlab = "Absolute Risk of Breast Cancer",
           main = "Referent SNP-only Risk Distribution: Ages 50-80 years")
abline(v = res_snps_dat$risk, col = "red")
legend("topright", legend = "New profiles", col = "red", lwd = 1)


###################################################
### code chunk number 7: ex-2
###################################################
res_covs_snps = computeAbsoluteRisk(model.formula = bc_model_formula,
                                model.cov.info = bc_model_cov_info,
                                model.snp.info = bc_72_snps,
                                model.log.RR = bc_model_log_or,
                                model.ref.dataset = ref_cov_dat,
                                model.disease.incidence.rates = bc_inc,
                                model.competing.incidence.rates = mort_inc,
                                model.bin.fh.name = "famhist",
                                apply.age.start = 50,
                                apply.age.interval.length = 30,
                                apply.cov.profile = new_cov_prof,
                                apply.snp.profile = new_snp_prof,
                                return.refs.risk = TRUE)


###################################################
### code chunk number 8: fit-2
###################################################
print(res_covs_snps$details)


###################################################
### code chunk number 9: sampling-weights
###################################################
validation.cohort.data$inclusion = 0
subjects_included = intersect(validation.cohort.data$id,
                            validation.nested.case.control.data$id)
validation.cohort.data$inclusion[subjects_included] = 1

validation.cohort.data$observed.followup =
                        validation.cohort.data$study.exit.age -
                            validation.cohort.data$study.entry.age

selection.model = glm(inclusion ~ observed.outcome
                       * (study.entry.age + observed.followup),
                              data = validation.cohort.data,
                              family = binomial(link = "logit"))
validation.nested.case.control.data$sampling.weights =
        selection.model$fitted.values[validation.cohort.data$inclusion == 1]


###################################################
### code chunk number 10: model-validation
###################################################
data = validation.nested.case.control.data
risk.model = list(model.formula = bc_model_formula,
                   model.cov.info = bc_model_cov_info,
                   model.snp.info = bc_72_snps,
                   model.log.RR = bc_model_log_or,
                   model.ref.dataset = ref_cov_dat,
                   model.ref.dataset.weights = NULL,
                   model.disease.incidence.rates = bc_inc,
                   model.competing.incidence.rates = mort_inc,
                   model.bin.fh.name = "famhist",
                   apply.cov.profile = data[,all.vars(bc_model_formula)[-1]],
                   apply.snp.profile = data[,bc_72_snps$snp.name],
                   n.imp = 5, use.c.code = 1, return.lp = TRUE,
                   return.refs.risk = TRUE)

output = ModelValidation(study.data = data,
                         total.followup.validation = TRUE,
                         predicted.risk.interval = NULL,
                         iCARE.model.object = risk.model,
                         number.of.percentiles = 10)


###################################################
### code chunk number 11: plot-model-validation
###################################################
plotModelValidation(study.data = data,validation.results = output)


###################################################
### code chunk number 12: sessionInfo
###################################################
sessionInfo()

ModelValidation = function(study.data,
                            total.followup.validation = FALSE,
                            predicted.risk = NULL, 
                            predicted.risk.interval = NULL, 
                            linear.predictor = NULL, 
                            iCARE.model.object = 
                              list(model.formula = NULL,
                                   model.cov.info = NULL,
                                   model.snp.info = NULL,
                                   model.log.RR = NULL,
                                   model.ref.dataset = NULL,
                                   model.ref.dataset.weights = NULL,
                                   model.disease.incidence.rates = NULL,
                                   model.competing.incidence.rates = NULL,
                                   model.bin.fh.name = NA,
                                   apply.cov.profile  = NULL,
                                   apply.snp.profile = NULL, 
                                   n.imp = 5, use.c.code = 1,
                                   return.lp = TRUE, 
                                   return.refs.risk = TRUE),
                            number.of.percentiles = 10,
                            reference.entry.age = NULL, 
                            reference.exit.age = NULL,
                            predicted.risk.ref = NULL,
                            linear.predictor.ref = NULL,
                            linear.predictor.cutoffs = NULL,
                            dataset = "Example Dataset", 
                            model.name = "Example Risk Prediction Model"){
  
  model.formula = iCARE.model.object$model.formula
  model.cov.info = iCARE.model.object$model.cov.info
  model.snp.info = iCARE.model.object$model.snp.info
  model.log.RR = iCARE.model.object$model.log.RR
  model.ref.dataset = iCARE.model.object$model.ref.dataset
  model.ref.dataset.weights = 
    iCARE.model.object$model.ref.dataset.weights
  model.disease.incidence.rates = 
    iCARE.model.object$model.disease.incidence.rates
  model.competing.incidence.rates = 
    iCARE.model.object$model.competing.incidence.rates
  model.bin.fh.name = iCARE.model.object$model.bin.fh.name
  apply.cov.profile = iCARE.model.object$apply.cov.profile
  apply.snp.profile = iCARE.model.object$apply.snp.profile
  n.imp = iCARE.model.object$n.imp
  use.c.code = iCARE.model.object$use.c.code
  return.lp = iCARE.model.object$return.lp
  return.refs.risk = iCARE.model.object$return.refs.risk
  
  ##########Some coding adjustment to create output##############
  mf = paste(model.formula)
  if(total.followup.validation){
    timeframe = "Observed Followup"
  } else timeframe = paste(predicted.risk.interval,"years")
  ###############################################################
  
  observed.outcome = study.data$observed.outcome
  study.entry.age = study.data$study.entry.age
  study.exit.age = study.data$study.exit.age
  time.of.onset = study.data$time.of.onset
  sampling.weights = study.data$sampling.weights
  
  if(any(study.entry.age >= study.exit.age)){
    print("Error: entry age should be lower than exit age")
    stop()
  }
  
  #Compute the observed followup
  observed.followup = study.exit.age - study.entry.age
  
  # cases with time to event bigger than observed followup (data issue!) are censored
  observed.outcome[observed.outcome == 1 & time.of.onset > observed.followup] = 0
  time.of.onset[observed.outcome == 1 & time.of.onset > observed.followup] = Inf
  
  #the followup variable is subsequently adjusted based on the risk prediction
  #interval
  followup = observed.followup
  
  #Setting up the risk prediction interval if total followup validation is
  #desired
  if(total.followup.validation){
    predicted.risk.interval = observed.followup
  }
  
  #If a scalar is supplied for predicted.risk.interval (e.g., 5), create a
  #vector
  if(length(predicted.risk.interval) == 1){
    predicted.risk.interval =
      rep(predicted.risk.interval,length(observed.outcome))
  }
  
  #if time of onset falls within the risk prediction interval, for model
  #validation we adjust the followup to be equal to the minimum of risk prediction
  #interval and observed followup
  
  followup[(time.of.onset <= predicted.risk.interval & predicted.risk.interval
            <= observed.followup)] = 
    predicted.risk.interval[(time.of.onset <= predicted.risk.interval &
                               predicted.risk.interval <= observed.followup)]                              
  
  #if the time of onset is larger than the length of risk prediction interval
  #we make the outcome censored at the right end point of risk prediction
  #interval
  observed.outcome[predicted.risk.interval < time.of.onset & time.of.onset <=
                     observed.followup] = 0
  followup[predicted.risk.interval < time.of.onset & time.of.onset <=
             observed.followup] =
    predicted.risk.interval[predicted.risk.interval < time.of.onset &
                              time.of.onset <= observed.followup]
  
  #if the observed followup is larger than the length of the risk prediction
  #interval and the subject has not developed disease by end of followup, she
  #is considered censored at the right endpoint of the risk prediction
  #interval
  followup[predicted.risk.interval <= observed.followup & observed.followup <
             time.of.onset] = predicted.risk.interval[predicted.risk.interval
                                                      <= observed.followup &
                                                        observed.followup <
                                                        time.of.onset]
  #if either predicted risk or linear predictors are not supplied estimate
  #both using iCARE
  if(is.null(predicted.risk) | is.null(linear.predictor)){
    
    pred.risk = computeAbsoluteRisk(model.formula = model.formula,
                                    model.cov.info = model.cov.info,
                                    model.snp.info = model.snp.info,
                                    model.log.RR = model.log.RR,
                                    model.ref.dataset = model.ref.dataset,
                                    model.ref.dataset.weights = NULL,
                                    model.disease.incidence.rates =
                                      model.disease.incidence.rates,
                                    model.competing.incidence.rates =
                                      model.competing.incidence.rates,
                                    model.bin.fh.name = model.bin.fh.name,
                                    apply.age.start = study.entry.age,
                                    apply.age.interval.length = followup,
                                    apply.cov.profile 
                                    = apply.cov.profile,
                                    apply.snp.profile = apply.snp.profile,
                                    use.c.code = use.c.code, return.lp =
                                      return.lp, return.refs.risk =
                                      return.refs.risk)
    
    predicted.risk = as.numeric(pred.risk$risk)
    linear.predictor = as.numeric(pred.risk$lps)
    
  }
  
  if((is.null(predicted.risk.ref) | is.null(linear.predictor.ref)) & 
     (!is.null(reference.entry.age)) & (!is.null(reference.exit.age))){
    
    reference.followup = reference.exit.age - reference.entry.age
    
    predicted.risk.ref = computeAbsoluteRisk(model.formula = model.formula,
                                             model.cov.info = model.cov.info,
                                             model.snp.info = model.snp.info,
                                             model.log.RR = model.log.RR,
                                             model.ref.dataset = 
                                               model.ref.dataset,
                                             model.ref.dataset.weights = NULL,
                                             model.disease.incidence.rates =
                                               model.disease.incidence.rates,
                                             model.competing.incidence.rates =
                                               model.competing.incidence.rates,
                                             model.bin.fh.name = 
                                               model.bin.fh.name,
                                             apply.age.start = 
                                               reference.entry.age,
                                             apply.age.interval.length 
                                             = reference.followup,
                                             apply.cov.profile = 
                                               model.ref.dataset,
                                             apply.snp.profile = 
                                               NULL,
                                             use.c.code = use.c.code, 
                                             return.lp = return.lp,
                                             return.refs.risk =
                                               return.refs.risk)
    
    linear.predictor.ref = as.numeric(predicted.risk.ref$lps)
    predicted.risk.ref = as.numeric(predicted.risk.ref$refs.risk)
  
  }
  
  linear.predictor.cases = linear.predictor[observed.outcome == 1]
  linear.predictor.controls = linear.predictor[observed.outcome == 0]
  
  #matrix of indicators of risk score in cases bigger than risk score in 
  #controls
  indicator = vapply(linear.predictor.cases, function(x) x > 
                       linear.predictor.controls,
                     logical(length(linear.predictor.controls)))
  
  #In nested case-control study setting there will be a vector of
  #"sampling.weights"; in the full cohort study setting, "sampling.weights"
  #is set to NULL
  
  if(!is.null(sampling.weights)){
    
    freq = 1/sampling.weights
    
    #Computing Incidence Rates
    ages = (min(study.entry.age) + 1):(max(study.exit.age) - 1)
    len_ages = length(ages)
    age.of.onset = study.entry.age + time.of.onset
    
    study_incidence = rep(0,len_ages)
    
    for (i in 1:len_ages) {
      
      a=sum((study.entry.age <= ages[i] - 1 & age.of.onset >= ages[i] & 
               age.of.onset < ages[i] + 1 & study.exit.age >= ages[i]) * 
              freq)
      b=sum((study.entry.age <= ages[i] - 1 & study.exit.age >= ages[i] & 
               age.of.onset >= ages[i]) * freq)
      
      study_incidence[i] = a/b
      
    }
    
    study_incidence = cbind(ages,study_incidence)
    
    sampling.weights.cases = sampling.weights[observed.outcome == 1]
    sampling.weights.controls = sampling.weights[observed.outcome == 0]
    
    #computing the weight matrix for the IPW estimate of AUC
    freq.cases = freq[observed.outcome == 1]
    freq.controls = freq[observed.outcome == 0]
    
    weight.mat = matrix(kronecker(freq.controls,freq.cases), 
                        nrow = length(freq.controls), byrow = TRUE)
    auc = sum(indicator * weight.mat)/sum(weight.mat)
    
    #compute E_S_0[I(S_1 > S_0)]
    mean_S0_indicator = apply(indicator,2,weighted.mean,w = freq.controls)
    
    #compute E_S_1[I(S_1 > S_0)]
    mean_S1_indicator = apply(indicator,1,weighted.mean,w = freq.cases)
    
    #compute the variance of AUC
    term1 = wtd.var(mean_S0_indicator, weights = freq.cases) + 
      weighted.mean((mean_S0_indicator - auc)^2 * 
                      (1 - sampling.weights.cases)/sampling.weights.cases, 
                    w = freq.cases)
    
    term2 = wtd.var(mean_S1_indicator, weights = freq.controls) + 
      weighted.mean((mean_S1_indicator - auc)^2 * 
                      (1 - sampling.weights.controls)/sampling.weights.controls, 
                    w = freq.controls)
    
    var_auc = term1/sum(freq.cases) + term2/sum(freq.controls)
    
    #Overall expected by observed ratio
    p_E = (sum(predicted.risk * freq))/(sum(freq))
    p_O = (sum(observed.outcome * freq))/(sum(freq))
    
    #variance of observed risk
    var_obsrisk = (p_O * (1 - p_O) + sum((observed.outcome - p_O)^2 * 
                  (1 - sampling.weights)/sampling.weights^2)/sum(freq))/sum(freq)
    
    #variance of log expected by observed
    var_log_exp_by_obs = var_obsrisk/p_O^2
    
    exp_by_obs = p_E/p_O
    
    CI_exp_by_obs = exp(c(log(exp_by_obs) - 1.96 * sqrt(var_log_exp_by_obs),
                        log(exp_by_obs) + 1.96 * sqrt(var_log_exp_by_obs)))
    
    #create categories of the risk score
    
    if(!is.null(linear.predictor.cutoffs)){
      
      linear.predictor.cutoffs = c(min(linear.predictor),
                                   linear.predictor.cutoffs,
                                   max(linear.predictor))
      
      linear.predictor.cat = cut(linear.predictor, 
                                 breaks = linear.predictor.cutoffs, 
                                 include.lowest = TRUE)
    
    } else {
      linear.predictor.cat = weighted.quantcut(linear.predictor, weights =
                                               freq, q =
                                               number.of.percentiles)
    }
    
    #compute sample size at each category using weights
    wtd.samp.size.cat = aggregate(freq ~ linear.predictor.cat, FUN = 
                                    sum)$freq
    
    #Observed and predicted probabilities at the categories
    obs.prob.wt = observed.outcome * freq
    
    obs.prob.wt.cat = aggregate(obs.prob.wt ~ linear.predictor.cat, FUN =
                                  sum)
    wt.cat = aggregate(freq ~ linear.predictor.cat, FUN = sum)
    
    observed.prob.cat = obs.prob.wt.cat$obs.prob.wt/wt.cat$freq
    
    pred.prob.wt = predicted.risk * freq
    
    pred.prob.wt.cat = aggregate(pred.prob.wt ~ linear.predictor.cat, FUN =
                                   sum)
    
    predicted.prob.cat = pred.prob.wt.cat$pred.prob.wt/wt.cat$freq
    
    #create a variable that assigns each subject to his category specific 
    #observed 
    #risk the variable is "observed.risk.cat" and is of same length as the 
    #number of subjects
    
    observed.risk.cat = linear.predictor.cat  
    levels(observed.risk.cat) = observed.prob.cat
    observed.risk.cat = as.numeric(as.character(observed.risk.cat))
    
    #compute the variance correction term of absolute risk for each 
    #category
    var.absrisk.corr = (observed.outcome - observed.risk.cat)^2 * 
      (1 - sampling.weights)/(sampling.weights)^2
    
    var.absrisk.corr.cat = (aggregate(var.absrisk.corr ~ 
                                        linear.predictor.cat, FUN = 
                                        sum)$var.absrisk.corr)/wt.cat$freq
    
    #compute the variance of absolute risk in each category
    variance.absrisk.cat = (observed.prob.cat * (1 - observed.prob.cat) +
                              var.absrisk.corr.cat)/wt.cat$freq
    
    #compute the standard deviation of absolute risk in each cateogry
    sd.absrisk.cat = sqrt(variance.absrisk.cat)
    
    #compute the Hosmer-Lemeshow statistic for absolute risk
    chisq.absrisk = sum(((observed.prob.cat - predicted.prob.cat)^2)
                        /variance.absrisk.cat)
    
    #sigma2_bar = mean(variance.absrisk.cat)
    mu_bar = mean(observed.prob.cat)
    
    #Producing relative risk estimates
    
    observed.RR.cat = observed.prob.cat/mu_bar
    predicted.RR.cat = predicted.prob.cat/mean(predicted.prob.cat)
    
    #Compute the variance-covariance matrix for the absolute risks
    variance.mat.absrisk = diag(variance.absrisk.cat)
    
    #Compute the derivative matrix for log relative risk
    derivative_offdiagonal = -1/(number.of.percentiles * mu_bar)
    derivative_diagonal = diag(1/observed.prob.cat + derivative_offdiagonal)
    
    derivative_diagonal[lower.tri(derivative_diagonal)] = derivative_offdiagonal
    derivative_diagonal[upper.tri(derivative_diagonal)] = derivative_offdiagonal
    
    derivative.mat = derivative_diagonal[-number.of.percentiles,]
    
    varcov.logRR.cat = derivative_diagonal %*% variance.mat.absrisk %*% 
      derivative_diagonal
    
    sd.logRR.cat = sqrt(diag(varcov.logRR.cat))
    
    sigmainv_logRR = solve(derivative.mat %*% variance.mat.absrisk %*% 
                             t(derivative.mat))
    
    diff.logRR = (log(observed.RR.cat) - 
                    log(predicted.RR.cat))[-number.of.percentiles]
    
    chisq.logRR = as.numeric(diff.logRR %*% sigmainv_logRR %*% t(t(diff.logRR)))
    
    #p-value for Hosmer Lemeshow test
    PVAL_absrisk = 1 - pchisq(chisq.absrisk, number.of.percentiles) 
    
    #p-value for relative risk test
    PVAL_logRR = 1 - pchisq(chisq.logRR, number.of.percentiles - 1) 
    
    #Compute 95% confidence interval for observed absolute risk in each 
    #category
    upper.limit.absrisk.cat = observed.prob.cat + 1.96 * sd.absrisk.cat
    lower.limit.absrisk.cat = observed.prob.cat - 1.96 * sd.absrisk.cat
    
    #Compute 95% confidence interval for relative risk in each category
    upper.limit.RR.cat = exp(log(observed.RR.cat) + 1.96 * sd.logRR.cat)
    lower.limit.RR.cat = exp(log(observed.RR.cat) - 1.96 * sd.logRR.cat)
    
    #Compute 95% confidence interval for the AUC estimate
    upper.limit.auc = auc + 1.96 * sqrt(var_auc)
    lower.limit.auc = auc - 1.96 * sqrt(var_auc)
    
  } else {
    
    print("No sampling weights provided: assuming each subject has sampling weight 1")
    
    #Compute incidence rates
    
    ages = (min(study.entry.age) + 1):(max(study.exit.age) - 1)
    len_ages = length(ages)
    age.of.onset = study.entry.age + time.of.onset
    
    study_incidence = rep(0,len_ages)
    
    for (i in 1:len_ages) {
      
      a=sum(study.entry.age <= ages[i] - 1 & age.of.onset >= ages[i] 
            & age.of.onset < ages[i] + 1 & study.exit.age >= ages[i])
      b=sum(study.entry.age <= ages[i] - 1 & study.exit.age >= ages[i] 
            & age.of.onset >= ages[i])
      
      study_incidence[i] = a/b
      
    }
    
    study_incidence = cbind(ages,study_incidence)
    
    #estimate the AUC in full cohort setting
    auc = mean(indicator)
    
    #compute E_S_0[I(S_1 > S_0)]
    mean_S0_indicator = apply(indicator,2,mean)
    
    #compute E_S_1[I(S_1 > S_0)]
    mean_S1_indicator = apply(indicator,1,mean)
    
    #compute variance of AUC
    var_auc = var(mean_S0_indicator)/length(mean_S0_indicator) + 
      var(mean_S1_indicator)/length(mean_S1_indicator)
    
    #Overall expected by observed ratio
    p_E = mean(predicted.risk)
    p_O = mean(observed.outcome)
    
    #variance of observed risk
    var_obsrisk = (p_O * (1 - p_O))/length(predicted.risk)
    
    #variance of log expected by observed
    var_log_exp_by_obs = var_obsrisk/p_O^2
    
    exp_by_obs = p_E/p_O
    
    CI_exp_by_obs = exp(c(log(exp_by_obs) - 1.96 * sqrt(var_log_exp_by_obs),
                        log(exp_by_obs) + 1.96 * sqrt(var_log_exp_by_obs)))
    
    #create categories of the risk score
    if(!is.null(linear.predictor.cutoffs)){
      
      linear.predictor.cutoffs = c(min(linear.predictor),
                                   linear.predictor.cutoffs,
                                   max(linear.predictor))
      
      linear.predictor.cat = cut(linear.predictor, 
                                 breaks = linear.predictor.cutoffs, 
                                 include.lowest = TRUE)
    
    } else {
      
      linear.predictor.cat = quantcut(linear.predictor, 
                                      q = number.of.percentiles)
    }
    
    #compute sample size at each category
    samp.size.cat = as.numeric(table(linear.predictor.cat))
    
    #Observed and predicted probabilities at the categories
    observed.prob.cat = aggregate(observed.outcome ~ linear.predictor.cat,
                                  FUN = mean)$observed.outcome
    predicted.prob.cat = aggregate(predicted.risk ~ linear.predictor.cat,
                                   FUN = mean)$predicted.risk
    
    #compute the variance in each category
    variance.absrisk.cat = (observed.prob.cat * 
                              (1 - observed.prob.cat))/samp.size.cat
    
    #compute the standard deviation in each cateogry
    sd.absrisk.cat = sqrt(variance.absrisk.cat)
    
    #compute the Hosmer-Lemeshow statistic
    chisq.absrisk = sum(((observed.prob.cat - predicted.prob.cat)^2)
                        /variance.absrisk.cat)
    
    #sigma2_bar = mean(variance.absrisk.cat)
    mu_bar = mean(observed.prob.cat)
    
    #Producing relative risk estimates
    
    observed.RR.cat = observed.prob.cat/mu_bar
    predicted.RR.cat = predicted.prob.cat/mean(predicted.prob.cat)
    
    #Compute the variance-covariance matrix for the absolute risks
    variance.mat.absrisk = diag(variance.absrisk.cat)
    
    #Compute the derivative matrix for log relative risk
    derivative_offdiagonal = -1/(number.of.percentiles * mu_bar)
    derivative_diagonal = diag(1/observed.prob.cat + derivative_offdiagonal)
    
    derivative_diagonal[lower.tri(derivative_diagonal)] = derivative_offdiagonal
    derivative_diagonal[upper.tri(derivative_diagonal)] = derivative_offdiagonal
    
    derivative.mat = derivative_diagonal[-number.of.percentiles,]
    
    varcov.logRR.cat = derivative_diagonal %*% variance.mat.absrisk %*% 
      derivative_diagonal
    
    sd.logRR.cat = sqrt(diag(varcov.logRR.cat))
    
    sigmainv_logRR = solve(derivative.mat %*% variance.mat.absrisk %*% 
                             t(derivative.mat))
    
    diff.logRR = (log(observed.RR.cat) - 
                    log(predicted.RR.cat))[-number.of.percentiles]
    
    chisq.logRR = as.numeric(diff.logRR %*% sigmainv_logRR %*% t(t(diff.logRR)))
    
    #p-value for Hosmer Lemeshow test
    PVAL_absrisk = 1 - pchisq(chisq.absrisk, number.of.percentiles)  
    
    #p-value for relative risk test
    PVAL_logRR = 1 - pchisq(chisq.logRR, number.of.percentiles - 1) 
    
    #Compute 95% confidence interval for observed absolute risk in each 
    #category
    upper.limit.absrisk.cat = observed.prob.cat + 1.96 * sd.absrisk.cat
    lower.limit.absrisk.cat = observed.prob.cat - 1.96 * sd.absrisk.cat
    
    #Compute 95% confidence interval for relative risk in each category
    upper.limit.RR.cat = exp(log(observed.RR.cat) + 1.96 * sd.logRR.cat)
    lower.limit.RR.cat = exp(log(observed.RR.cat) - 1.96 * sd.logRR.cat)
    
    #Compute 95% confidence interval for the AUC estimate
    upper.limit.auc = auc + 1.96 * sqrt(var_auc)
    lower.limit.auc = auc - 1.96 * sqrt(var_auc)
    
  }
  
  #Output results
  DNAME <- paste(deparse(substitute(observed.frequency)),
                 deparse(substitute(expected.frequency)), 
                 sep = ", ")
  METHOD = "Hosmer and Lemeshow goodness of fit (GOF) test for Absolute Risk"
  names(chisq.absrisk) = "Chisquare"
  names(number.of.percentiles) = "df"
  
  h1 = structure(list(statistic = chisq.absrisk, parameter = 
                       number.of.percentiles, 
                     p.value = PVAL_absrisk, method = METHOD, 
                     data.name = DNAME),class = "htest")
  
  #cat("\n")
  
  DNAME <- paste(deparse(substitute(observed.frequency)),
                 deparse(substitute(expected.frequency)), 
                 sep = ", ")
  METHOD = "Goodness of fit (GOF) test for Relative Risk"
  names(chisq.logRR) = "Chisquare"
  names(number.of.percentiles) = "df"
  
  h2 = structure(list(statistic = chisq.logRR, parameter = 
                       number.of.percentiles - 1, 
                     p.value = PVAL_logRR, method = METHOD, data.name = DNAME), 
                 class = "htest")
  
  #cat("\n")
  
  #print(paste("Dataset: ",dataset))
  #print(paste("Model Name: ",model.name))
  #if(is.null(model.formula)){
  #  print("Model formula: Likely an additive SNP-only model")
  #} else {
  #print(paste("Model Formula:",mf[2],mf[1],mf[3]))
  #}
  #print(paste("Risk Prediction Interval:",timeframe))
  #cat("\n")
  #print(paste("Number of study subjects: "
  #                               ,length(observed.outcome)))
  #cat("\n")
  #print(paste("Number of cases: ",sum(observed.outcome)))
  #cat("\n")
  #print(paste("Follow-up time (years) [mean,range]: [",
  #                    round(mean(followup),3),", (",
  #                    round(range(followup)[1],3),",",
  #                    round(range(followup)[2],3),")"," ]"))
  #cat("\n")
  #print(paste("Baseline age (years) [mean,range]: [", 
  #            round(mean(study.entry.age),3),", (",
  #            round(range(study.entry.age)[1],3),",",
  #            round(range(study.entry.age)[2],3),")"," ]"))
  #cat("\n")
  #print("Absolute Risk Calibration")
  #print(h1)
  #cat("\n")
  #print("Relative Risk Calibration")
  #print(h2)
  #cat("\n")
  #print("Model Discrimination")
  #print(paste("Estimate of AUC:",round(auc,3)))
  #print(paste("95% CI of AUC: (",round(lower.limit.auc,3),",",
  #            round(upper.limit.auc,3),")"))
  #cat("\n")
  #print("Overall Expected to Observed Ratio")
  #print(paste("Estimate:",round(exp_by_obs,3)))
  #print(paste("95% CI:","(",round(CI_exp_by_obs[1],3),
  #            ",",round(CI_exp_by_obs[2],3),")"))
  
  
  results.cat = data.frame(cbind(levels(linear.predictor.cat),
                                 observed.prob.cat, predicted.prob.cat,
                                 lower.limit.absrisk.cat, upper.limit.absrisk.cat,
                                 observed.RR.cat, predicted.RR.cat,
                                 lower.limit.RR.cat, upper.limit.RR.cat
                                 ))
  
  names(results.cat) = c("Categories","Observed_Absolute_Risk","Predicted_Absolute_Risk",
                         "CI_Absolute_Risk_Lower","CI_Absolute_Risk_Upper",
                         "Observed_Relative_Risk","Predicted_Relative_Risk",
                         "CI_Relative_Risk_Lower","CI_Relative_Risk_Upper")
  
  ret <- list(Subject_Specific_Observed_Outcome = observed.outcome,
              Risk_Prediction_Interval = timeframe,
              Adjusted_Followup = followup, 
              Subject_Specific_Predicted_Absolute_Risk = predicted.risk, 
              Reference_Absolute_Risk = predicted.risk.ref,
              Subject_Specific_Risk_Score = linear.predictor, 
              Reference_Risk_Score = linear.predictor.ref,
              Population_Incidence_Rate = model.disease.incidence.rates,
              Study_Incidence_Rate = study_incidence,
              Category_Results = results.cat, 
              Category_Specific_Observed_Absolute_Risk = observed.prob.cat,
              Category_Specific_Predicted_Absolute_Risk = 
                predicted.prob.cat,
              Category_Specific_Observed_Relative_Risk = observed.RR.cat,
              Category_Specific_Predicted_Relative_Risk = predicted.RR.cat,
              Variance_Matrix_Absolute_Risk = variance.mat.absrisk,
              Variance_Matrix_LogRelative_Risk = varcov.logRR.cat,
              Hosmer_Lemeshow_Results = h1, 
              HL_pvalue = PVAL_absrisk, RR_test_result = h2, 
              RR_test_pvalue = PVAL_logRR,
              AUC = auc, Variance_AUC = var_auc, 
              CI_AUC = c(lower.limit.auc,upper.limit.auc),
              Overall_Expected_to_Observed_Ratio = exp_by_obs,
              CI_Overall_Expected_to_Observed_Ratio = CI_exp_by_obs,
              input.args=list(model.formula=model.formula, study.data=study.data,
                 dataset=dataset, model.name=model.name))
  
  class(ret) <- "icareValid"
  ret
}

weighted.quantcut = function (x, weights = NULL, q = 10, na.rm = TRUE, ...){
    
    if (length(q) == 1){ 
        q <- seq(0, 1, length.out = q + 1)
    }
    
    quant <- wtd.quantile(x, weights, q, type = "i/n", normwt = FALSE, 
                          na.rm = na.rm)
    dups <- duplicated(quant)
    if (any(dups)) {
        flag <- x %in% unique(quant[dups])
        retval <- ifelse(flag, paste("[", as.character(x), "]", 
            sep = ""), NA)
        uniqs <- unique(quant)

        reposition <- function(cut) {
            flag <- x >= cut
            if (sum(flag) == 0) 
                return(cut)
            else return(min(x[flag], na.rm = na.rm))
        }
        newquant <- sapply(uniqs, reposition)
        retval[!flag] <- as.character(cut(x[!flag], breaks = newquant, 
            include.lowest = TRUE, ...))
        levs <- unique(retval[order(x)])
        retval <- factor(retval, levels = levs)
        mkpairs <- function(x) sapply(x, function(y) if (length(y) == 
            2) 
            y[c(2, 2)]
        else y[2:3])
        levs  <- levs[!is.na(levs)]
        pairs <- mkpairs(strsplit(levs, "[^0-9+\\.\\-]+"))
        rownames(pairs) <- c("lower.bound", "upper.bound")
        colnames(pairs) <- levs
        closed.lower <- rep(F, ncol(pairs))
        closed.upper <- rep(T, ncol(pairs))
        closed.lower[1] <- TRUE
        for (i in 2:ncol(pairs)) if (pairs[1, i] == pairs[1, 
            i - 1] && pairs[1, i] == pairs[2, i - 1]) 
            closed.lower[i] <- FALSE
        for (i in 1:(ncol(pairs) - 1)) if (pairs[2, i] == pairs[1, 
            i + 1] && pairs[2, i] == pairs[2, i + 1]) 
            closed.upper[i] <- FALSE
        levs <- ifelse(pairs[1, ] == pairs[2, ], pairs[1, ], 
            paste(ifelse(closed.lower, "[", "("), pairs[1, ], 
                ",", pairs[2, ], ifelse(closed.upper, "]", ")"), 
                sep = ""))
        levels(retval) <- levs
    }
    else retval <- cut(x, quant, include.lowest = TRUE, ...)
    return(retval)
}

plotModelValidation = function(study.data, validation.results,
                                 dataset = "Example Dataset",
                                 model.name = "Example Model",
                                 x.lim.absrisk = NULL,
                                 y.lim.absrisk = NULL, 
                                 x.lab.absrisk = "Expected Absolute Risk (%)", 
                                 y.lab.absrisk = "Observed Absolute Risk (%)", 
                                 x.lim.RR = NULL,
                                 y.lim.RR = NULL, 
                                 x.lab.RR = "Expected Relative Risk", 
                                 y.lab.RR = "Observed Relative Risk",
                                 risk.score.plot.kernel = "gaussian",
                                 risk.score.plot.bandwidth = "nrd0",
                                 risk.score.plot.percent.smooth = 50){
  
  Category.Results = validation.results$Category_Results
  followup = validation.results$Adjusted_Followup
  observed.outcome = validation.results$Subject_Specific_Observed_Outcome
  linear.predictor = validation.results$Subject_Specific_Risk_Score
  model.disease.incidence.rates = 
    validation.results$Population_Incidence_Rate
  timeframe = validation.results$Risk_Prediction_Interval
  ages = validation.results$Study_Incidence_Rate[,1]
  study_incidence = validation.results$Study_Incidence_Rate[,2]
  
  study.entry.age = study.data$study.entry.age
  sampling.weights = study.data$sampling.weights
  
  if(is.null(sampling.weights))
    sampling.weights = rep(1,dim(study.data)[1])
  
  freq = 1/sampling.weights
  
  freq.cases = freq[observed.outcome == 1]
  freq.controls = freq[observed.outcome == 0]
  
  linear.predictor.cases = 
    linear.predictor[observed.outcome == 1]
  
  linear.predictor.controls = 
    linear.predictor[observed.outcome == 0]
  
  observed.prob.cat = 
    as.numeric(as.character(Category.Results$Observed_Absolute_Risk))
  predicted.prob.cat = 
    as.numeric(as.character(Category.Results$Predicted_Absolute_Risk))
  
  observed.RR.cat = 
    as.numeric(as.character(Category.Results$Observed_Relative_Risk))
  predicted.RR.cat = 
    as.numeric(as.character(Category.Results$Predicted_Relative_Risk))
  
  upper.limit.absrisk.cat = 
    as.numeric(as.character(Category.Results$CI_Absolute_Risk_Upper))
  lower.limit.absrisk.cat = 
    as.numeric(as.character(Category.Results$CI_Absolute_Risk_Lower))
  
  upper.limit.RR.cat = 
    as.numeric(as.character(Category.Results$CI_Relative_Risk_Upper))
  lower.limit.RR.cat = 
    as.numeric(as.character(Category.Results$CI_Relative_Risk_Lower))
  
  chisq.absrisk = 
    as.numeric(validation.results$Hosmer_Lemeshow_Results[["statistic"]])
  number.of.percentiles = 
    as.numeric(validation.results$Hosmer_Lemeshow_Results[["parameter"]])
  PVAL_absrisk = 
    as.numeric(validation.results$Hosmer_Lemeshow_Results[["p.value"]])
  
  chisq.logRR = as.numeric(validation.results$RR_test_result[["statistic"]])
  PVAL_logRR = as.numeric(validation.results$RR_test_result[["p.value"]])
  
  auc = validation.results$AUC
  upper.limit.auc = validation.results$CI_AUC[2]
  lower.limit.auc = validation.results$CI_AUC[1]
  
  Overall.Expected.to.Observed.Ratio = 
    validation.results$Overall_Expected_to_Observed_Ratio
  CI.Overall.Expected.to.Observed.Ratio = 
    validation.results$CI_Overall_Expected_to_Observed_Ratio

  
  #adjust x and y axes to draw absolute risk calibration plot
    if(length(x.lim.absrisk) < 2){
      x.lim.absrisk = c(min(lower.limit.absrisk.cat, 
                            predicted.prob.cat)*100,
                        max(upper.limit.absrisk.cat, 
                            predicted.prob.cat)*100)
    }
    
    if(length(y.lim.absrisk) < 2){
      y.lim.absrisk = c(min(lower.limit.absrisk.cat, 
                            predicted.prob.cat)*100,
                        max(upper.limit.absrisk.cat, 
                            predicted.prob.cat)*100)
    }
    
    #adjust x and y axes to draw relative risk calibration plot
    if(length(x.lim.RR) < 2){
      x.lim.RR = c(min(lower.limit.RR.cat, predicted.RR.cat),
                   max(upper.limit.RR.cat, predicted.RR.cat))
    }
    
    if(length(y.lim.RR) < 2){
      y.lim.RR = c(min(lower.limit.RR.cat, predicted.RR.cat),
                   max(upper.limit.RR.cat, predicted.RR.cat))
    }
    
  m <- rbind(c(1,2),c(3,4),c(5,5))
  layout(m)
  
  oldpar =  par(mar = rep(4,4))
  plotCI(predicted.prob.cat*100, observed.prob.cat *100, ui =
           upper.limit.absrisk.cat * 100, 
         li = lower.limit.absrisk.cat * 100, 
         xlab = x.lab.absrisk, ylab = y.lab.absrisk, xlim = 
           x.lim.absrisk, 
         ylim = y.lim.absrisk, col = "red",
         pch = 16, pty = "s",cex.lab = 1.2)
  abline(0,1,lty=2,col="black")
  #mtext(paste("Hosmer-Lemeshow p-value:",PVAL),side = 3, line = 1)
  mtext("Absolute Risk Calibration",side = 3, line =
          1,font = 4)
  
  plotCI(predicted.RR.cat, observed.RR.cat, ui =
           upper.limit.RR.cat, li = lower.limit.RR.cat, xlab =
           x.lab.RR, ylab = y.lab.RR, xlim = x.lim.RR, ylim = y.lim.RR,
         col = "red",
         pch = 16, pty = "s",cex.lab = 1.2)
  abline(0,1,lty=2,col="black")
  #mtext(paste("Hosmer-Lemeshow p-value:",PVAL),side = 3, line = 1)
  mtext("Relative Risk Calibration",side = 3, line =
          1,font = 4)
  
  plot(density(linear.predictor.controls, 
               bw = risk.score.plot.bandwidth,
               kernel = risk.score.plot.kernel,
               weights = freq.controls/sum(freq.controls),
               n = (risk.score.plot.percent.smooth/100) * 
                 length(linear.predictor.controls)),
       xlim = c(min(density(linear.predictor.controls)$x,
                    density(linear.predictor.cases)$x),
                max(density(linear.predictor.controls)$x,
                    density(linear.predictor.cases)$x)),
       ylim = c(min(density(linear.predictor.controls)$y,
                    density(linear.predictor.cases)$y),
                max(density(linear.predictor.controls)$y,
                    density(linear.predictor.cases)$y)),
       main = "",xlab = "Risk Score", pty = "s",cex.lab = 1.2)
  lines(density(linear.predictor.cases,
                bw = risk.score.plot.bandwidth,
                kernel = risk.score.plot.kernel,
                weights = freq.cases/sum(freq.cases),
                n = (risk.score.plot.percent.smooth/100) * 
                 length(linear.predictor.controls)),col = "red")
  legend("topright",legend = c("Controls","Cases"),
         col = c("black","red"),
         lty=1,cex = 0.75,y.intersp = 2, xjust = 0.5, 
         yjust = 0.5, x.intersp = 3,adj = c(0.3,0.6))
  mtext("Discrimination",side = 3, line =
          1,font = 4)
  
  plot(model.disease.incidence.rates[,1],
       predict(loess(model.disease.incidence.rates[,2] 
                     ~ model.disease.incidence.rates[,1])),
       log = "y", type = "l",
       xlim = c(18,max(model.disease.incidence.rates[,1])),
       ylim = c(min(model.disease.incidence.rates[,2],study_incidence) + 
                  10^(-10),
                max(model.disease.incidence.rates[,2],study_incidence)),
       xlab = "Age (in years)",ylab = "Incidence Rates",pty = "s",
       cex.lab = 1.2)
  lines(ages,study_incidence,col = "red")
  legend("bottomright",legend = c("Population","Study"),
         col = c("black","red"),lty=1,
         cex = 0.9,y.intersp = 2, x.intersp = 3,adj = c(0.3,0.6))
  mtext("Incidence Rates",side = 3, line =
          1,font = 4)
  
  plot(c(0,2),c(0,2),xaxt='n',yaxt='n',bty='n',pch='',ylab='',xlab='',
       pty = "s")
  text(x = 1, y = 2, paste("Dataset: ",dataset))
  text(x = 1, y = 1.8, paste("Model Name: ",model.name))
  text(x = 1, y = 1.6, paste("Risk Prediction Interval:",timeframe))
  text(x = 1, y = 1.4, paste("Number of subjects (cases): "
                             ,length(observed.outcome),"(",
                             sum(observed.outcome),")"))
  text(x = 1, y = 1.2, paste("Follow-up time (years) [mean,range]: [",
                               round(mean(followup),3),", (",
                               round(range(followup)[1],3),",",
                               round(range(followup)[2],3),")"," ]"))
  text(x = 1, y = 1, paste("Baseline age (years) [mean,range]: [", 
                               round(mean(study.entry.age),3),", (",
                               round(range(study.entry.age)[1],3),",",
                               round(range(study.entry.age)[2],3),")"," ]"))
  text(x = 1, y = 0.8, paste("E/O [Estimate, 95% CI]:"
                             ,"[",round(Overall.Expected.to.Observed.Ratio,3),
                             ",","(",round(CI.Overall.Expected.to.Observed.Ratio[1],3),
                             ",",round(CI.Overall.Expected.to.Observed.Ratio[2],3),
                             ")","]"))
  text(x = 0.3, y = 0.5, "Absolute Risk Calibration", font = 2)
  text(x = 0.3, y = 0.3, paste("HL Test, df:",
                               round(chisq.absrisk,3),",",number.of.percentiles))
  #text(x = 0.1, y = 0.3, paste("HL Test df:",number.of.percentiles))
  text(x = 0.3, y = 0.1, paste("p-value:",
                               noquote(ifelse(PVAL_absrisk < 
                                                1e-100,"<1e-100",
                                              format.pval(PVAL_absrisk,digits=20, 
                                                          eps = 1e-100, 
                                                          scientific = TRUE)))))
  text(x = 1, y = 0.5, "Relative Risk Calibration", font = 2)
  text(x = 1, y = 0.3, paste("Test, df:",
                               round(chisq.logRR,3),",",number.of.percentiles - 1))
  #text(x = 0.5, y = 0.3, paste("Test df:",number.of.percentiles - 1))
  text(x = 1, y = 0.1, paste("p-value:",
                               noquote(ifelse(PVAL_logRR < 1e-100,"<1e-100",
                                              format.pval(PVAL_logRR,digits=20,
                                                         eps = 1e-100, 
                                                          scientific = 
                                                           TRUE)))))
  text(x = 1.7, y = 0.5, "Model Discrimination", font = 2)
  text(x = 1.7, y = 0.3, paste("AUC est:",round(auc,3)))
  text(x = 1.7, y = 0.1, 
       paste("95% CI: (",round(lower.limit.auc,3),",",
             round(upper.limit.auc,3),")"))
  
  par(oldpar)
  
  NULL
}

# lambda should be matrix of two columns (age, incidence rates ) diff ages in the rows

pick_lambda <- function(t, lambda){

  a <- which(t==lambda[,1])
  lambda[a,2]
}

# c should be a two column matrix with first column times

# computes internal integral over u for estimate of A_j
get_int <- function(a,t,lambda, Z_new, beta_est, model.competing.incidence.rates, ZBETA = NULL){

  holder<-0
  if (is.null(ZBETA)) {
    ZBETA <- t(exp(t(Z_new)%*%beta_est))
  }

  for(u in min(a, na.rm = TRUE):max(t, na.rm = TRUE)){
    temp0  <- (u>=a)*(u<t)
    holder <- holder + temp0*( (pick_lambda(u,lambda))*ZBETA + model.competing.incidence.rates[which(u==model.competing.incidence.rates[,1]),2])
  }
  holder
}

get_beta_given_names <-function(model.cov.info, model.formula){

  check_triple_check(model.cov.info)
  model.cov.info = process_triple_check(model.cov.info)

  if(is.null(colnames(model.cov.info))){
    return_and_print("ERROR: model.cov.info must have same names and order as predictors in model.formula.")
    stop()
  }
  if(length(colnames(model.cov.info))!=length(all.vars(model.formula)[2:length(all.vars(model.formula))])){
    return_and_print("ERROR: model.cov.info must have same names and order as predictors in model.formula.")
    stop()
  }
  if( sum(colnames(model.cov.info)!=all.vars(model.formula)[2:length(all.vars(model.formula))])>0 ){
    return_and_print("ERROR: model.cov.info must have same names and order as predictors in model.formula.")
    stop()
  }
  variables <- unique(all.vars(model.formula))[-1]
  data_use  <- subset(model.cov.info, select = variables)
  data_use[,all.vars(model.formula)[1]] <- rep(0, nrow(data_use))
  predictors <- as.matrix(model.matrix(model.formula, data = as.data.frame(data_use))[,2:ncol(model.matrix(model.formula, data = as.data.frame(data_use)))])
  colnames(predictors)

}

###  make design matrix for apply.cov.profile
make_design_matrix_covs <- function(data_use, apply.cov.profile, model.formula){

  options(na.action='na.pass')
  init_n   <- nrow(data_use)
  jump_cov <- nrow(apply.cov.profile )

  data_use   <- rbind(data_use, apply.cov.profile )
  data_use[,all.vars(model.formula)[1]] <- rep(0, nrow(data_use))

  predictors <- as.matrix(model.matrix(model.formula, data = as.data.frame(data_use))[,2:ncol(model.matrix(model.formula, data = as.data.frame(data_use)))])
  set        <- rep(FALSE, nrow(predictors))
  set[(init_n+1):(init_n+jump_cov)]=TRUE
  apply.cov.profile     <- subset( predictors, subset = set)
  apply.cov.profile
}

###  make design matrix for model.ref.dataset
make_design_matrix_dist <- function(data_use, model.ref.dataset, model.formula){

  init_n   <- nrow(data_use)
  jump_cov <- nrow(model.ref.dataset)

  data_use <- rbind(data_use, model.ref.dataset)
  data_use <- cbind(data_use, rep(0, nrow(data_use)))
  colnames(data_use) <- c(colnames(data_use)[1:(length(colnames(data_use))-1)],  all.vars(model.formula)[1])

  predictors   <- as.matrix(model.matrix(model.formula, data = as.data.frame(data_use))[,2:ncol(model.matrix(model.formula, data = as.data.frame(data_use)))])
  pop.dist.mat <- predictors[(init_n+1):(init_n+jump_cov),]
  cbind(pop.dist.mat)
}

###### Compute Absolute Risk
comp_Aj <- function(Z_new, apply.age.start, apply.age.interval.length, lambda, beta_est, model.competing.incidence.rates){

    ZBETA <- t(exp(t(beta_est)%*%Z_new))
    avec  <- apply.age.start + apply.age.interval.length
    tvec  <- min(apply.age.start, na.rm = TRUE):max(avec, na.rm = TRUE)
    temp  <- match(tvec, lambda[,1])
    c2    <- t(Z_new)%*%beta_est
    c3    <- log(lambda[temp, 2])

    Aj_est <- rep(0, ncol(Z_new))
    for(i in 1:length(tvec)){
      t         <- tvec[i]
      temp      <- get_int(apply.age.start,t,lambda, Z_new, beta_est, model.competing.incidence.rates, ZBETA = ZBETA)
      this_year <- (t >= apply.age.start)*(t < avec)*exp(c3[i] + c2 - temp)

      Aj_est    <- Aj_est + this_year
    }
    Aj_est
}

### iterate to obtain lambda_0
precise_lambda0 <- function(lambda, approx_expectation_rr, beta_est, pop.dist.mat, pop.weights){
  diagnose                <- list()
  lambda_0                <- lambda
  precise_expectation_rr0 <- approx_expectation_rr-1
  precise_expectation_rr1 <- approx_expectation_rr
  iter                    <- 0
  while( sum(abs(precise_expectation_rr1 - precise_expectation_rr0 )) >0.001 ){

    iter = iter+1
    precise_expectation_rr0 = precise_expectation_rr1
    diagnose[[iter]]=precise_expectation_rr0
    # new expectation rr implies lambda0
    lambda_0[,2] = lambda[,2] / precise_expectation_rr0
    # that lambda0 implies new expectation rr
    # this should be Nobs x Ntimes
    this = survival_givenX(lambda_0, beta_est, pop.dist.mat)*matrix(nrow=nrow(pop.dist.mat), ncol=nrow(lambda_0), pop.weights, byrow=TRUE)
    denom = 1/colSums(this)
    probX_givenT = sweep( this, MARGIN = 2, denom, FUN="*" )

    # to compute next iteration
    precise_expectation_rr1 = colSums( sweep( probX_givenT, MARGIN = 1,  exp( pop.dist.mat %*% beta_est ), FUN="*") )
  }
  lambda_0[,2] = lambda[,2] / precise_expectation_rr1
  res = list();  res[[1]] = lambda_0;  res[[2]] = precise_expectation_rr1; res[[3]]=diagnose
  res
}

### getting lambda_0 iteratively
survival_givenX <- function(lambda_0, beta_est, pop.dist.mat){ # must produce matrix with Nobs x Ntimes

  ## cumulative up to but not including current time
  mult   <- -exp( pop.dist.mat %*% beta_est )
  cumLam <- cumsum( c(0,lambda_0[,2]) )[1:length(lambda_0[,2])]
  exp( mult %*% cumLam  )
}

## transform model.cov.info into a form easily usable in the function
process_triple_check <- function(model.cov.info){
  maximum_dim = 0
  for(i in 1:length(model.cov.info)){
    maximum_dim = max(maximum_dim, length(model.cov.info[[i]]$levels) )
  }
  matt = data.frame(starter_column = rep(0, maximum_dim))

  for(i in 1:length(model.cov.info)){

    if(model.cov.info[[i]]$type == "continuous"){
        matt[,model.cov.info[[i]]$name] = rep(0,maximum_dim)
    }
    if(model.cov.info[[i]]$type == "factor"){
        matt[,model.cov.info[[i]]$name] = factor( c(model.cov.info[[i]]$levels, rep(model.cov.info[[i]]$levels[1], maximum_dim - length(model.cov.info[[i]]$levels))), levels = unique(c(model.cov.info[[i]]$levels, rep(model.cov.info[[i]]$levels[1], maximum_dim - length(model.cov.info[[i]]$levels)))))
        ### if there is a referent level specified, recode that way
        if(is.null(model.cov.info[[i]]$ref)==FALSE){
            matt[,model.cov.info[[i]]$name] = relevel(matt[,model.cov.info[[i]]$name], ref = as.character(model.cov.info[[i]]$ref))
        }
    }
  }
  matt[,1] <- NULL
  matt
}

## helper function
## creates model.cov.info from a dataframe
## option to flag factor variables by including a vector of Variable names
## if factor_vars input is not provided, then considered continuous if more than 12 unique levels, else factor
make_triple_check <- function(frame, factor_vars = NULL){

  if( !is.data.frame(frame) ){
    return_and_print("ERROR: input must be a dataframe")
    stop()
  }

  the_names = colnames(frame)
  model.cov.info = list()
  length(model.cov.info)=length(the_names)

  if(is.null(factor_vars)==FALSE){
    if(is.vector(factor_vars)!=TRUE ){
      return_and_print(paste("ERROR: The optional 'flag' of factor variables must be specified as a vector.", sep="") )
      stop()
    }
    if( prod( is.element(factor_vars, the_names)) == 0 ){
      return_and_print(paste("ERROR: The optional 'flag' of factor variables must contain names that match the column names of the dataframe.", sep="") )
      stop()
    }
  }

  for(i in 1:ncol(frame)){

    temp = list()
    temp$name = the_names[i]
    ###
    if(is.null(factor_vars)==FALSE){
      toggle = !is.element(temp$name, factor_vars)
    }
    if(is.null(factor_vars)){
      # binary indicator of whether the variable is continuous
      toggle = length(c(unique(frame[,i])))>12
      if( is.factor(frame[,13]) ==TRUE ){
        toggle = FALSE
      }
    }###
    if(toggle){
      temp$type = "continuous"
    }
    if(toggle == FALSE){
      temp$type = "factor"
      temp$levels = c(unique(frame[,i]))
      temp$ref = temp$levels[1]
    }
    model.cov.info[[i]]=temp
  }
  model.cov.info
}

## Function for combining Absolute Risks for Adjacent Intervals
combine_risks_two_intervals <- function(age_start_1, age_interval_1, absolute_risks_1, age_start_2, age_interval_2,
                                        absolute_risks_2){

  if( sum(age_start_1 + age_interval_1 != age_start_2) >0 ){
    return_and_print(paste("ERROR: To combine risks for two intervals, they must be adjacent.", sep="") )
    stop()
  }

  overall_age_start = age_start_1
  overall_age_interval = age_interval_1 + age_interval_2
  overall_absolute_risks =  absolute_risks_1 + (1 - absolute_risks_1)*absolute_risks_2

  result = list()
  result$details = cbind(overall_age_start, overall_age_interval, overall_absolute_risks)
  result$risk = cbind(overall_absolute_risks)
  result
}

## helper function for computing absolute risk of competing mortality
comp_AR_compete <- function(apply.age.start, apply.age.interval.length, model.competing.incidence.rates){
  ###### Compute A_j

  Aj_est <- 0
  for(t in min(apply.age.start, na.rm = TRUE):max(apply.age.start+apply.age.interval.length, na.rm = TRUE)){

    this_year <- (t>=apply.age.start)*(t<apply.age.start+apply.age.interval.length)*exp( log(pick_lambda(t, model.competing.incidence.rates)) - get_int_compete(apply.age.start,t, model.competing.incidence.rates) )#ex[t-apply.age.start+1]=Aj_est
    Aj_est    <- Aj_est + this_year
  }
  Aj_est
}

## helper function for computing survival of competing risk
get_int_compete <- function(a,t,model.competing.incidence.rates){

  holder <- 0
  for(u in min(a, na.rm = TRUE):max(t, na.rm = TRUE)){
    holder <- holder+(u>=a)*(u<t)*model.competing.incidence.rates[which(u==model.competing.incidence.rates[,1]),2]
  }
  holder
}

sim_snps <- function(betas, freqs, fh_status){

  snps <- matrix( NA, ncol = length(betas), nrow = length(fh_status))

  prob012_fh_no = cbind( (1-freqs)^2, 2*freqs*(1-freqs), freqs^2)

  beta_mat = matrix( betas, nrow = length(betas), ncol = 3, byrow = FALSE)
  top = exp( beta_mat*matrix(c(0,1,2), nrow = length(betas), ncol = 3, byrow = TRUE)/2 )*matrix(prob012_fh_no, nrow = length(betas), ncol = 3, byrow = FALSE)
  bottom = matrix(rowSums(top), nrow =length(betas), ncol = 3, byrow = FALSE)

  prob012_fh_yes = top/bottom

  fh_no  <- which(fh_status==0)
  fh_yes <- which(fh_status==1)

  vals = matrix( runif(length(betas)*length(fh_status)), ncol = length(betas), nrow = length(fh_status))

  if(length(fh_no)>0){
    snps[fh_no,] =  ( vals[fh_no,] > matrix(prob012_fh_no[,1], nrow = length(fh_no), ncol = length(prob012_fh_no[,1]), byrow = TRUE ) ) + ( vals[fh_no,] > matrix(rowSums(subset(prob012_fh_no, select=1:2)), nrow = length(fh_no), ncol = length(prob012_fh_no[,1]), byrow = TRUE ) )
  }
  if(length(fh_yes)>0){
    snps[fh_yes,] =  ( vals[fh_yes,] > matrix(prob012_fh_yes[,1], nrow = length(fh_yes), ncol = length(prob012_fh_yes[,1]), byrow = TRUE ) ) + ( vals[fh_yes,] > matrix(rowSums(subset(prob012_fh_yes, select=1:2)), nrow = length(fh_yes), ncol = length(prob012_fh_yes[,1]), byrow = TRUE ) )
  }
  snps

}

## helper function for handling missing data
handle_missing_data <- function(use.c.code, apply.age.start, apply.age.interval.length, Z_new, miss, present, ncuts, final_risks,
                                ref_pop, pop.weights, lambda_0, beta_est, model.competing.incidence.rates, lps){
  
  ref_full_LP <- t(ref_pop)%*%beta_est
  ###### Handle Missing Data  ##### If All times are the same
  if( length(unique(apply.age.start[miss]))==1 & length(unique(apply.age.interval.length[miss]))==1){
    
    pop.apply.age.start = unique(apply.age.start)  # change to single values so don't have to worry about dimension of ref_pop
    pop.apply.age.interval.length = unique(apply.age.interval.length)

    ###### Compute A_j_pop for ref_risks

    ref_risks   <- comp_Aj(ref_pop, pop.apply.age.start, pop.apply.age.interval.length, lambda_0, beta_est, model.competing.incidence.rates)

    if (use.c.code) {
      temp <- call_c1(final_risks, lps, ref_full_LP, miss, ref_risks, ref_pop, beta_est[, 1],
                             Z_new, pop.weights, ncuts=ncuts, debug = 0)
      final_risks <- temp$final ## will be: final_risks  <- temp[[1]]
      lps         <- temp$lps      ## will be: lps <- temp[[2]]
    } else {
      BETA  <- t(beta_est)
      Z_NEW <- Z_new
      REF   <- ref_pop
      PROBS <- seq(0, 1, 1/ncuts)

      for(i in 1:length(miss)){
        missi <- miss[i]

        ## make sure LPs based on non-missing covariates for the observation with missing
        present     <- which(is.na(Z_NEW[, missi])!=TRUE)
        if(length(present)==0){
          final_risks[missi] <- 0 
        }else{
        BETAV       <- BETA[1, present, drop = FALSE]
        ref_LP      <- BETAV%*%REF[present, , drop = FALSE] #just LP from predictors?
        miss_LP     <- BETAV%*%Z_NEW[present, missi, drop = FALSE] #just LP from predictors?

        cutpoints           <- get_cutpoints(ref_LP, PROBS)
        ncuts               <- length(cutpoints)
        miss_perc           <- get_miss_perc(cutpoints, miss_LP)
        these               <- (ref_LP >= cutpoints[miss_perc]) & (ref_LP < cutpoints[miss_perc+1])
        these[is.na(these)] <- FALSE
        if (!sum(these)) {
          temp                <- get_newEndpoints(cutpoints, miss_LP, miss_perc, ref_LP)
          these               <- (ref_LP >= temp[1]) & (ref_LP < temp[2])
          these[is.na(these)] <- FALSE
        }
        final_risks[missi]  <- weighted.mean(ref_risks[these], w = pop.weights[these], na.rm = TRUE)
        lps[missi]          <- weighted.mean(ref_full_LP[these], w = pop.weights[these], na.rm = TRUE)
        }
      }
    }
  } # END: if( length(unique(apply.age.start[miss]))==1 & length(unique(apply.age.interval.length[miss]))==1)


  ###### Handle Missing Data  ##### If All times are different
  if( length(unique(apply.age.start[miss])) > 1 || length(unique(apply.age.interval.length[miss])) > 1){
    if (use.c.code) {
      popSubFromLP = rep(0, ncol(ref_pop))
      temp  <- call_c2(final_risks, lps, ref_full_LP, miss, ref_pop, beta_est[, 1], Z_new, apply.age.start,
                               apply.age.interval.length, popSubFromLP, lambda_0, model.competing.incidence.rates, pop.weights,
                               ncuts = ncuts, debug = 0)
      final_risks <- temp$final ## will be: final_risks  <- temp[[1]]
      lps         <- temp$lps      ## will be: lps <- temp[[2]]
    } else {

      BETA  <- t(beta_est)
      Z_NEW <- Z_new
      REF   <- ref_pop
      CVEC  <- 2:(ncuts-1)
      PROBS <- seq(0, 1, 1/ncuts)

      for(i in 1:length(miss)){
        missi <- miss[i]

        pop.apply.age.start = apply.age.start[missi]  # change to single values so don't have to worry about dimension of ref_pop
        pop.apply.age.interval.length = apply.age.interval.length[missi];

        ## make sure LPs based on non-missing covariates for the observation with missing
        present = which(is.na(Z_new[,missi])!=TRUE)
        if(length(present)==0){
          final_risks[missi] <- 0 
        }else{
        BETAV       <- BETA[1, present, drop = FALSE]
        ref_LP      <- BETAV%*%REF[present, , drop = FALSE] #just LP from predictors?
        miss_LP     <- BETAV%*%Z_NEW[present, missi, drop = FALSE] #just LP from predictors?

        cutpoints           <- get_cutpoints(ref_LP, PROBS)
        ncuts               <- length(cutpoints)
        miss_perc           <- get_miss_perc(cutpoints, miss_LP)
        these               <- (ref_LP >= cutpoints[miss_perc]) & (ref_LP < cutpoints[miss_perc+1])
        these[is.na(these)] <- FALSE
        if (!sum(these)) {
          temp                <- get_newEndpoints(cutpoints, miss_LP, miss_perc, ref_LP)
          these               <- (ref_LP >= temp[1]) & (ref_LP < temp[2])
          these[is.na(these)] <- FALSE
        }
        temp                <- REF[,these, drop = FALSE]
        ref_risks           <- comp_Aj(temp, pop.apply.age.start, pop.apply.age.interval.length,
                                       lambda_0, beta_est, model.competing.incidence.rates)
        final_risks[missi]  <- weighted.mean(ref_risks, w = pop.weights[these], na.rm = TRUE)
        lps[missi]          <- weighted.mean(ref_full_LP[these], w = pop.weights[these], na.rm = TRUE)
      }}
    }
  }
  ## end function
  res = list(); res[[1]] = final_risks; res[[2]] = lps; res
}

decide_if_SNP_only <- function(apply.cov.profile , model.formula, model.log.RR, model.ref.dataset, model.cov.info,
                               model.snp.info, apply.snp.profile, apply.age.start, apply.age.interval.length){

  if(is.null(apply.cov.profile ) & is.null(model.formula) & is.null(model.log.RR) & is.null(model.ref.dataset) & is.null(model.cov.info)){
    covs_in_model = 0
    if( is.null(model.snp.info) ){
      return_and_print("ERROR: You are fitting a SNP-only Model, and thus must provide the 'model.snp.info' object.")
      stop()
    }
    if(is.null(apply.snp.profile)){
      if(length(apply.age.start)==1 & length(apply.age.interval.length)==1){
        apply.snp.profile = matrix(ncol = nrow(model.snp.info), nrow = 10000, NA)
        cat("\nNote: You did not provide apply.snp.profile.  Will impute SNPs for 10000 people.\n")
        cat("If require more, please provide apply.snp.profile input.\n")
      }else{
        apply.snp.profile = matrix(ncol = nrow(model.snp.info), nrow = length(apply.age.start), NA)
        print(paste("Note: You did not provide apply.snp.profile.  Imputed SNPs for ", length(apply.age.start) , " individuals, matching number of age intervals specified.", sep=""))
      }
      }
    if(is.vector(apply.snp.profile)){
      apply.snp.profile = cbind(apply.snp.profile)
     }
  }else{

    if( is.null(apply.cov.profile ) + is.null(model.formula) + is.null(model.log.RR) + is.null(model.ref.dataset) + is.null(model.cov.info) >0  ){
      return_and_print("ERROR: If any of apply.cov.profile , model.formula, model.log.RR, model.ref.dataset, or model.cov.info are NULL then they must all be NULL, and will define a SNP-only Model.")
      stop()
    }
    if(is.vector(apply.snp.profile)){
      apply.snp.profile = cbind(apply.snp.profile)
    }
    covs_in_model = 1
  }
  res <- list(); res[[1]] = covs_in_model; res[[2]] = apply.snp.profile; res
}

process_SNP_info <- function(covs_in_model, apply.snp.profile, model.bin.fh.name, apply.cov.profile , model.ref.dataset, model.snp.info){
  if( covs_in_model){

    if(is.null(apply.snp.profile)){
      apply.snp.profile = matrix(ncol = nrow(model.snp.info), nrow = nrow(apply.cov.profile ), NA)
      print("Note: You included snp_info, but did not provide apply.snp.profile.  Will impute all SNPs. ")
    }
    if(nrow(apply.snp.profile)!=nrow(apply.cov.profile )){
      return_and_print("ERROR: apply.cov.profile  and apply.snp.profile must have same number of rows. ")
      stop()
    }
    if(!is.na(model.bin.fh.name)){
      if( !is.element( model.bin.fh.name, colnames(apply.cov.profile )) ){
        return_and_print("ERROR: model.bin.fh.name must contain the variable name of family history (matching a column name in apply.cov.profile ) if it is in the model, otherwise NA.")
        stop()
      }else{
        fh.pop = model.ref.dataset[,model.bin.fh.name]
        fh.cov = apply.cov.profile [,model.bin.fh.name]
        attenuate.fh = 1
        if( prod(is.element(fh.pop, c(0,1,"0", "1", NA)) ) == 0){
          return_and_print("ERROR: The family history must be binary when using snp_info functionality. Check input for model.ref.dataset.")
          stop()
        }
        if( prod(is.element(fh.cov, c(0,1,"0", "1", NA)) ) == 0){
          return_and_print("ERROR: The family history must be binary when using snp_info functionality. Check input for apply.cov.profile .")
          stop()
        }}
    }else{ # model.bin.fh.name = NA - family history not in the model
      fh.pop = rep(0, nrow( model.ref.dataset))
      fh.cov = rep(0, nrow( apply.cov.profile ))
      attenuate.fh = 0
      print("Note: As specified, the model does not adjust SNP imputations for family history, as model.bin.fh.name = NA.")
    }
  }else{
    fh.pop = rep(0, 10000)
    fh.cov = rep(0, nrow( apply.snp.profile))
    attenuate.fh = 0
    print("Note: As specified, the model does not adjust SNP imputations for family history.")
  }
  res <- list(); res[[1]] = attenuate.fh; res[[2]] = fh.pop; res[[3]] = fh.cov; res[[4]] = apply.snp.profile; res
}

#' Building and Applying an Absolute Risk Model
#'
#' This function is used to build absolute risk models and apply them to estimate absolute risks.
#'
#' @param ref.pop design matrix created from dataframe of risk factors for a sample of subjects representative of underlying population, no missing values.
#' @param apply.age.start single integer or vector of integer ages for the start of the interval over which to compute absolute risk.
#' @param apply.age.interval.length single integer or vector of integer years over which absolute risk should be computed.
#' @param lambda_0 two column matrix [ integer ages, incidence rates] with incidence rate of disease. Must fully cover age interval for estimation.
#' @param beta_est vector with log odds ratios corresponding to the model params; no intercept; names must match design matrix, \code{ref_pop}.
#' @param model.competing.incidence.rates two column matrix [ integer ages, incidence rates] with incidence rate of competing events. Must fully cover age interval for estimation.
#' @param handle.snps binary indicator of whether SNPs are in the design matrix or not.  If so, design matrix has been stacked 5 times.
#' @return This function returns risks for the referent risk factor dataset.
#' @export
#' @examples
#' print("Hello world")
get_refs_risk <- function(ref_pop, apply.age.start, apply.age.interval.length, lambda_0, beta_est, model.competing.incidence.rates, handle.snps, n.imp ){

  refs.risk = comp_Aj(ref_pop, apply.age.start[1], apply.age.interval.length[1], lambda_0, beta_est, model.competing.incidence.rates)
  refs.lps = t(ref_pop)%*%beta_est
  
  if(handle.snps){
    refs.risk = rowMeans( matrix(refs.risk, ncol=n.imp, nrow=length(refs.risk)/n.imp, byrow=FALSE))
    refs.lps  = rowMeans( matrix(refs.lps,  ncol=n.imp, nrow=length(refs.lps)/n.imp, byrow=FALSE))
  }
  res = list(); res[[1]] = refs.risk; res[[2]] = refs.lps ; res
}


return_and_print <- function(x){
  print(x)
  return(x)  
}

test_get_int_compete <- function() {

  lambda <- matrix(1:20, nrow=10, byrow=FALSE)
  rates  <- lambda
  ret    <- iCARE:::get_int_compete(1,2,rates)

  checkEqualsNumeric(ret, 11, tolerance=1.0e-8)
  
}

test_get_int <- function() {

  lambda <- matrix(1:20, nrow=10, byrow=FALSE)
  ZBETA  <- 0.5
  rates  <- lambda
  ret    <- iCARE:::get_int(1,2,lambda, NULL, NULL, rates, ZBETA=ZBETA)

  checkEqualsNumeric(ret, 16.5, tolerance=1.0e-8)
  
}

### R code from vignette source 'vignette.Rnw'

###################################################
### code chunk number 1: start
###################################################
library(iCARE)


###################################################
### code chunk number 2: load-data
###################################################
data("bc_data", package="iCARE")


###################################################
### code chunk number 3: model-1a
###################################################
res_snps_miss = computeAbsoluteRisk(model.snp.info = bc_72_snps, 
                      model.disease.incidence.rates = bc_inc,
                      model.competing.incidence.rates = mort_inc, 
                      apply.age.start = 50, 
                      apply.age.interval.length = 30,
                      return.refs.risk=TRUE)


###################################################
### code chunk number 4: summary-1a
###################################################
summary(res_snps_miss$risk)
summary(res_snps_miss$refs.risk)
plot(density(res_snps_miss$risk), lwd=2, 
  main="SNP-only Risk Stratification: Ages 50-80", 
  xlab="Absolute Risk of Breast Cancer")


###################################################
### code chunk number 5: model 1b
###################################################
res_snps_dat = computeAbsoluteRisk(model.snp.info = bc_72_snps, 
                                model.disease.incidence.rates = bc_inc,
                                model.competing.incidence.rates = mort_inc, 
                                apply.age.start = 50, 
                                apply.age.interval.length = 30, 
                                apply.snp.profile = new_snp_prof, 
                                return.refs.risk = TRUE)
names(res_snps_dat)


###################################################
### code chunk number 6: plot-1b
###################################################
plot(density(res_snps_dat$refs.risk), lwd=2, 
   main="Referent SNP-only Risk Distribution: Ages 50-80",
   xlab="Absolute Risk of Breast Cancer")
abline(v=res_snps_dat$risk, col="red")
legend("topright", legend="New Profiles", col="red", lwd=1)


###################################################
### code chunk number 7: fit-2
###################################################
res_covs_snps = computeAbsoluteRisk(model.formula=bc_model_formula, 
                                         model.cov.info=bc_model_cov_info,
                                         model.snp.info=bc_72_snps,
                                         model.log.RR=bc_model_log_or,
                                         model.ref.dataset=ref_cov_dat,
                                         model.disease.incidence.rates=bc_inc,
                                         model.competing.incidence.rates=mort_inc, 
                                         model.bin.fh.name="famhist",
                                         apply.age.start=50, 
                                         apply.age.interval.length=30,
                                         apply.cov.profile=new_cov_prof,
                                         apply.snp.profile=new_snp_prof, 
                                         return.refs.risk=TRUE)


###################################################
### code chunk number 8: fit-2
###################################################
print(res_covs_snps$details)


###################################################
### code chunk number 9: sessionInfo
###################################################
sessionInfo()

## verify that model.cov.info is in the proper form
check_triple_check <- function(model.cov.info){

  if( is.list(model.cov.info)!=TRUE){
    return_and_print("ERROR: model.cov.info must be a list.")
    stop()
  }
  for(i in 1:length(model.cov.info)){
    if( is.list(model.cov.info[[i]])!=TRUE ){
      return_and_print("ERROR: Each element of model.cov.info must be a list (that contains the information for a variable). ")
      stop()
    }
    if( is.null(model.cov.info[[i]]$name) ){
      return_and_print("ERROR: Each list within model.cov.info describing a variable must have a 'name' field. ")
      stop()
    }
    if( is.null(model.cov.info[[i]]$type) ){
      return_and_print("ERROR: Each list within model.cov.info describing a variable must have a 'type' field. ")
      stop()
    }}
  for(i in 1:length(model.cov.info)){

    if( model.cov.info[[i]]$type == "continuous"  & is.null(model.cov.info[[i]]$levels)!=TRUE){
      print(paste("WARNING: You have specified that the variable '", model.cov.info[[i]]$name, "' is continuous, so the 'levels' input for that variable is not needed and will not be used.", sep="") )
    }# allow to continue
    if( model.cov.info[[i]]$type == "factor"  & is.null(model.cov.info[[i]]$levels)==TRUE){
      return_and_print(paste("ERROR: You must specify the 'levels' field of the variable '", model.cov.info[[i]]$name, "', because you gave its type to be 'factor'.", sep="") )
      stop()
    }
    if( model.cov.info[[i]]$type == "factor"  & is.null(model.cov.info[[i]]$levels)==FALSE){
      if(is.vector(model.cov.info[[i]]$levels)!=TRUE ){
        return_and_print(paste("ERROR: You must specify the 'levels' field of the variable '", model.cov.info[[i]]$name, "', as a vector.", sep="") )
        stop()
      }
      if(is.null(model.cov.info[[i]]$ref)==FALSE){
        if(is.element(model.cov.info[[i]]$ref, model.cov.info[[i]]$levels)==FALSE){
          return_and_print(paste("ERROR: The 'ref' field of the variable '", model.cov.info[[i]]$name, "' must specify a referent level that is contained in the 'levels' field for that variable.", sep="") )
          stop()
        }
      }
    }
  }
}

check_disease_rates <- function(filename){

  lambda = read.table(filename, sep=",", header=TRUE)
  lambda = check_flexible_rate_inputs(lambda, "model.disease.incidence.rates")
  lambda
}

check_competing_rates <- function(filename, lambda){

  if(is.null(filename) || filename==""){
    model.competing.incidence.rates= data.frame( cbind(lambda[,1], rep(0, length(lambda[,1]))) )
  }else{
    model.competing.incidence.rates = read.table(filename, sep=",", header=TRUE)
  }
  model.competing.incidence.rates = check_flexible_rate_inputs(model.competing.incidence.rates, "model.competing.incidence.rates")
  model.competing.incidence.rates
}

## function to convert (start, end, rate) matrix into (integer_ages, rate) matrix
format_flexible_rate_inputs<-function(mat){

  if(ncol(mat)==3){
    start <- mat[,1]
    end   <- mat[,2]
    rate  <- mat[,3]

    integer_ages <- seq(min(start), max(end))
    formatted    <- cbind( integer_ages, rep(NA, length(integer_ages))  )
    for(i in 1:nrow(mat)){
      these              <- which(formatted[,1]>=start[i] & formatted[,1]<=end[i])
      formatted[these,2] <- rate[i]/length(these)
    }
    colnames(formatted) <- c("ages", "rates")
    formatted
    }else{
      formatted = mat
    }
  formatted
}

check_flexible_rate_inputs<-function(mat, name){
  if(!is.data.frame(mat) && !is.matrix(mat)){
    return_and_print(paste("ERROR: ", name, " must be provided as a matrix.", sep=""))
    stop()
  }
  if(ncol(mat)!=3 & ncol(mat)!=2){
    return_and_print(paste("ERROR: ", name, " must have either 2 columns: [Ages,Rates] or 3 columns:[Start_Ages, End_Ages, Rates].", sep=""))
    stop()
  }
  ll = nrow(mat)
  if(ll>1 & ncol(mat)==3){
    if(sum(mat[2:ll,1] - mat[1:(ll-1),2])!=0){
      return_and_print(paste("ERROR: The rates provided in ", name, " must cover sequential age intervals (i.e. if an interval ends at age 30, the next interval must start at age 31).", sep=""))
      stop()
    }
  }
  if(ncol(mat)==2 && sum(mat[,1]%%1)!=0){
    return_and_print(paste("ERROR: The first column of ", name, " should be integer ages.", sep=""))
    stop()
  }
  if(sum(mat[,ncol(mat)]<0, na.rm=TRUE) + sum(mat[,ncol(mat)]>1, na.rm=TRUE)!=0){
    return_and_print("ERROR: The rates should be probabilities between 0 and 1.")
    stop()
  }
  format_flexible_rate_inputs(mat)
}

check_rates <- function(model.competing.incidence.rates, lambda, apply.age.start, apply.age.interval.length){

  lambda = check_flexible_rate_inputs(lambda, "model.disease.incidence.rates")

  if(is.null(model.competing.incidence.rates)){  model.competing.incidence.rates= cbind(lambda[,1], rep(0, length(lambda[,1])))   }

  model.competing.incidence.rates = check_flexible_rate_inputs(model.competing.incidence.rates, "model.competing.incidence.rates")

  if( prod(is.element(seq(range(apply.age.start)[1], range(apply.age.start + apply.age.interval.length)[2]), lambda[,1])) == 0){
    return_and_print("ERROR: The 'model.disease.incidence.rates' input must have age-specific rates for each integer age covered by the prediction intervals defined by 'apply.age.start' and 'apply.age.interval.length.'  You must make these inputs consistent with one another to proceed.")
    stop()
  }
  if( prod(is.element(seq(range(apply.age.start)[1], range(apply.age.start + apply.age.interval.length)[2]), model.competing.incidence.rates[,1])) == 0){
    return_and_print("ERROR: The 'model.competing.incidence.rates' input must have age-specific rates for each integer age covered by the prediction intervals defined by 'apply.age.start' and 'apply.age.interval.length.'  You must make these inputs consistent with one another to proceed.")
    stop()
  }
  res=list(); res[[1]] = lambda; res[[2]] = model.competing.incidence.rates; res
}

check_model_inputs <- function(apply.cov.profile , model.log.RR, model.ref.dataset, model.ref.dataset.weights, model.cov.info, model.formula, n.imp){
  if(length(n.imp)!=1){
    return_and_print("ERROR: n.imp must be a single integer.")
    stop()
  }
  if( (n.imp < 1) || (n.imp > 20) ){
    return_and_print("ERROR: n.imp must be a single integer between 0 and 20.")
    stop()
  }
  if(!is.data.frame(apply.cov.profile ) && !is.matrix(apply.cov.profile )){
    return_and_print("ERROR: apply.cov.profile  must be a dataframe.")
    stop()
  }
  if(!is.data.frame(model.ref.dataset)&& !is.matrix(model.ref.dataset)){
    return_and_print("ERROR: model.ref.dataset must be a dataframe (with appropriate column names).")
    stop()
  }
  if(sum(rowSums(is.na(model.ref.dataset)))>0){
    return_and_print("ERROR: model.ref.dataset cannot contain NAs.")
    stop()
  }
  check_triple_check(model.cov.info)
  model.cov.info = process_triple_check(model.cov.info)

  ### Will verify all names and proper order of things against model.cov.info - so check against model first
  if(is.null(colnames(model.cov.info))){
    return_and_print("ERROR: model.cov.info must have same names and order as predictors in model.formula.")
    stop()
  }
  if(length(colnames(model.cov.info))!=length(all.vars(model.formula)[2:length(all.vars(model.formula))])){
    return_and_print("ERROR: model.cov.info must have same names and order as predictors in model.formula.")
    stop()
  }
  if( sum(colnames(model.cov.info)!=all.vars(model.formula)[2:length(all.vars(model.formula))])>0 ){
    return_and_print("ERROR: model.cov.info must have same names and order as predictors in model.formula.")
    stop()
  }
  if(prod(colnames(model.cov.info)==colnames(apply.cov.profile ))!=1){
    return_and_print("ERROR: Variable names of model.cov.info should match column names of apply.cov.profile .")
    stop()
  }
  for(k in 1:ncol(model.cov.info)){
    if(is.factor(model.cov.info[,k])){
      if(sum(is.element(apply.cov.profile [,k], model.cov.info[,k])+(is.na(c(apply.cov.profile [,k])))==0)>0){
        return_and_print(paste("ERROR: apply.cov.profile  categorical Variable (", colnames(model.cov.info)[k] ,") has levels outside levels modeled."))
        stop()
      }
    }
  }
  if(prod(colnames(model.cov.info)==colnames(model.ref.dataset))!=1){
    return_and_print("ERROR: Variable names of model.cov.info should match column names of model.ref.dataset.")
    stop()
  }
  if(nrow(model.ref.dataset) < 200){
    print(paste("WARNING: Samples in referent distribution model.ref.dataset should be large.  Currently only size ", nrow(model.ref.dataset),".", sep=""))
  }
  if(is.null(model.ref.dataset.weights)){
    model.ref.dataset.weights = rep(1/nrow(model.ref.dataset), nrow(model.ref.dataset))
  }else{
    if(length(model.ref.dataset.weights)!=nrow(model.ref.dataset)){
      return_and_print("ERROR: If model.ref.dataset.weights is provided it must be same length as the number of rows in model.ref.dataset.")
      stop()
    }
    if( sum((model.ref.dataset.weights<0), na.rm=TRUE)!=0){
      return_and_print("ERROR: model.ref.dataset.weights must not contain negative values.")
      stop()
    }
    model.ref.dataset.weights = model.ref.dataset.weights / sum(model.ref.dataset.weights, na.rm=TRUE)
  }
  ## for now don't allow NA's here
  for(k in 1:ncol(model.cov.info)){
    if(is.factor(model.cov.info[,k])){
      if(sum(is.element(c(model.ref.dataset[,k]), model.cov.info[,k])==0)>0){
        return_and_print(paste("ERROR: model.ref.dataset categorical Variable (", colnames(model.cov.info)[k] ,") has levels outside levels modeled."))
        stop()
      }
    }
  }
  variables <- unique(all.vars(model.formula))[-1]
  data_use  <- subset(model.cov.info, select = variables)

  if(is.vector(model.log.RR)==FALSE & is.matrix(model.log.RR)==FALSE){
    return_and_print("ERROR: model.log.RR must be either a vector, or a matrix with only one column.")
    stop()
  }
  if(is.matrix(model.log.RR)==TRUE){
    if(dim(model.log.RR)[2]>1){
      return_and_print("ERROR: model.log.RR must be either a vector, or a matrix with only one column.")
      stop()
    }
  }
  if(is.vector(model.log.RR)==TRUE){
    temp = names(model.log.RR)
    model.log.RR = cbind(model.log.RR)
    rownames(model.log.RR) = temp
  }
  res = list(); res[[1]] = data_use; res[[2]] = model.log.RR; res[[3]] = model.ref.dataset.weights; res
}

check_design_matrix <- function(model.log.RR, design_covs){

    if(is.null(rownames(model.log.RR)) ){
      return_and_print("ERROR: row names of model.log.RR must match names and order in design matrix.")
      print(paste("Row names of model.log.RR = NULL"))
      print(paste("Names of design_matrix = ", paste0(colnames(design_covs), collapse=", ") ))
      stop()
    }
    if( sum( rownames(model.log.RR)!= colnames(design_covs) )>0){
      return_and_print("ERROR: row names of model.log.RR must match names and order in design matrix.")
      print(cbind(names(model.log.RR), colnames(design_covs)))
      stop()
    }
}

check_age_inputs <- function(apply.age.start, apply.age.interval.length, apply.cov.profile, apply.snp.profile, lambda, model.competing.incidence.rates){

  if(!is.null(apply.cov.profile)){
    temp <- check_age_lengths(apply.age.start, apply.age.interval.length, apply.cov.profile , "apply.cov.profile ")
  }else if(!is.null(apply.snp.profile)){
    temp <- check_age_lengths(apply.age.start, apply.age.interval.length, apply.snp.profile, "apply.snp.profile")
  }else{
    return_and_print("ERROR: Must provide either apply.cov.profile or apply.snp.profile, consistent with model.")
    stop()
  }

  if(mode(temp)=="character"){
    return_and_print(temp)
    stop()
  }else{
    apply.age.start           <- temp[[1]]
    apply.age.interval.length <- temp[[2]]

    if( prod(is.element(seq(range(apply.age.start)[1], range(apply.age.start + apply.age.interval.length)[2]), lambda[,1])) == 0){
      return_and_print("ERROR: The prediction intervals defined by 'apply.age.start' and 'apply.age.interval.length' are not covered by the provided 'model.disease.incidence.rates' input. You must make these inputs consistent with one another to proceed.")
      stop()
    }
    if( prod(is.element(seq(range(apply.age.start)[1], range(apply.age.start + apply.age.interval.length)[2]), model.competing.incidence.rates[,1])) == 0){
      return_and_print("ERROR: The prediction intervals defined by 'apply.age.start' and 'apply.age.interval.length' are not covered by the provided 'model.competing.incidence.rates' input. You must make these inputs consistent with one another to proceed.")
      stop()
    }}
  NULL
}

check_age_lengths <- function(apply.age.start, apply.age.interval.length, match, match_name){

  if(length(apply.age.start)==1){
    apply.age.start = rep(apply.age.start, nrow(match))
  }
  if(length(apply.age.interval.length)==1){
    apply.age.interval.length = rep(apply.age.interval.length, nrow(match))
  }
  if(length(apply.age.start)!=length(apply.age.interval.length)){
    return_and_print("ERROR: apply.age.start and apply.age.interval.length must have the same length.")
    stop()
  }
  if(length(apply.age.start) != nrow(match)){
    return_and_print(paste("ERROR: apply.age.start and number of rows of ", match_name," must match.", sep=""))
    stop()
  }
  if(sum(is.na(apply.age.start)+is.na(apply.age.interval.length))>0){
    return_and_print("ERROR: apply.age.start and apply.age.interval.length may not contain missing values.")
    stop()
  }
  if(sum((apply.age.start<0)+(apply.age.interval.length<0))>0){
    return_and_print("ERROR: apply.age.start and apply.age.interval.length must contain positive values.")
    stop()
  }
  res <- list(); res[[1]] = apply.age.start; res[[2]] = apply.age.interval.length; res
}

check_SNP_info <- function(model.snp.info){
  if( !is.data.frame(model.snp.info) && !is.matrix(model.snp.info) ){
    return_and_print("ERROR: If specified model.snp.info must be a dataframe (that contains the information on snp names, odds ratios, and allele frequencies. ")
    stop()
  }
  if( ncol(model.snp.info)!=3 ){
    return_and_print("ERROR: If specified, model.snp.info must be a dataframe with 3 columns named 'snp.name', 'snp.odds.ratio', and 'snp.freq'.")
    stop()
  }
  if( prod(is.element( names(model.snp.info), c("snp.name","snp.odds.ratio","snp.freq" )))!=1 ){
    return_and_print("ERROR: If specified, model.snp.info must be a dataframe with 3 columns named 'snp.name', 'snp.odds.ratio', and 'snp.freq'.")
    stop()
  }
}

# Function to compute cutpoints
get_cutpoints <- function(ref_LP, probs) {

  # Get the number of digits
  se     <- sd(ref_LP, na.rm = TRUE)
  if (se < 1e-12) se <- 1e-12
  prec   <- 1/(0.001*se)
  digits <- sum(prec >= 10^(1:16))
  cuts   <- quantile(ref_LP, probs = probs, names = FALSE)
  cuts   <- round_down(cuts, digits = digits) # new 9/1/15
  cuts   <- c(unique(cuts), Inf)# new 9/1/15
  if (length(cuts) < 2) stop("ERROR: computing cutpoints")

  cuts

} # END: get_cutpoints

round_down <-function(num, digits){
  
  floor( num*10^digits ) / (10^digits)
  
}

get_endpoints <- function(cutpoints, miss_LP) {

  ncuts     <- length(cutpoints)
  miss_perc <- sum(cutpoints <= as.numeric(miss_LP))
  if (miss_perc == ncuts) {
    miss_perc <- miss_perc - 1
  } else if (!miss_perc) {
    miss_perc <- 1
  }
  a <- cutpoints[miss_perc]
  b <- cutpoints[miss_perc+1]

  c(a, b)

} # END: get_endpoints

# Function to return missing percentile
get_miss_perc <- function(cutpoints, miss_LP) {

  ncuts     <- length(cutpoints)
  miss_perc <- sum(cutpoints <= as.numeric(miss_LP))
  if (miss_perc == ncuts) {
    miss_perc <- miss_perc - 1
  } else if (!miss_perc) {
    miss_perc <- 1
  }

  miss_perc

} # END: get_miss_perc

# Functon to return the endpoints
get_newEndpoints <- function(cutpoints, miss_LP, miss_perc, ref_LP) {

  # Remove cutpoint defined by miss_perc + 1
  ncuts <- length(cutpoints)
  left  <- miss_perc - 1
  right <- miss_perc + 2
  for (i in 1:ncuts) {
    if (left < 1) left <- 1
    if (right > ncuts) right <- ncuts
    cut1 <- cutpoints[left]
    cut2 <- cutpoints[right]
    temp <- (ref_LP >= cut1) & (ref_LP < cut2)
    temp[is.na(temp)] <- FALSE
    if (any(temp)) break
    left  <- left - 1
    right <- right + 1
  }

  c(cut1, cut2)

} # END: get_newEndpoints

test_get_newEndpoints <- function() {

  cutpoints <- 1:10
  ref_LP    <- 0:100
  miss_LP   <- 5.5
  probs     <- c(0, 0.25, 0.5, 0.75, 1)
  miss_perc <- 5

  ret       <- iCARE:::get_newEndpoints(cutpoints, miss_LP, miss_perc, ref_LP) 

  checkEquals(sum(ret == c(4, 7)), 2)
  
}

test_get_miss_perc <- function() {

  cutpoints <- 1:10
  ref_LP    <- 0:100
  miss_LP   <- 5.5
  probs     <- c(0, 0.25, 0.5, 0.75, 1)
  miss_perc <- 5

  ret       <- iCARE:::get_miss_perc(cutpoints, miss_LP) 

  checkEqualsNumeric(ret, 5, tolerance=1.0e-8)
  
}

%\VignetteIndexEntry{iCARE Vignette Model Validation}
%\VignettePackage{iCARE}
%\VigetteDepends{iCARE}

\documentclass[a4paper]{article}
\begin{document}
\SweaveOpts{concordance=TRUE}

\title{iCARE (Individualized Coherent Absolute Risk Estimation) Package}
\maketitle

%Install the iCARE package 
%<<start>>=
%if (!requireNamespace("BiocManager", quietly=TRUE))
%    install.packages("BiocManager")
%BiocManager::install("iCARE", version = 3.8)
%@

Load the iCARE library
<<start>>=
library(iCARE)
@

Load the breast cancer data and set the seed.
<<load-data>>=
data("bc_data", package="iCARE")
set.seed(50)
@

\section*{Example 1: SNP-only model}
In this example, we will estimate the risk of breast cancer in ages 50-80.
A SNP-only model is fit, with no specific genotypes supplied for estimation. 
The population disease rates are from SEER.
<<model-1a>>=
res_snps_miss = computeAbsoluteRisk(model.snp.info = bc_72_snps,
                       model.disease.incidence.rates = bc_inc,
                       model.competing.incidence.rates = mort_inc,
                       apply.age.start = 50, apply.age.interval.length = 30,
                       return.refs.risk = TRUE)
@
Compute a summary of the risks.
<<summary-1a>>=
summary(res_snps_miss$refs.risk)
@

Next, suppose we want to predict risk for three specific women whom we have genotyped; we can then call:
<<model 1b>>=
res_snps_dat = computeAbsoluteRisk(model.snp.info = bc_72_snps,
                     model.disease.incidence.rates = bc_inc,
                     model.competing.incidence.rates = mort_inc,
                     apply.age.start = 50, apply.age.interval.length = 30,
                     apply.snp.profile = new_snp_prof,
                     return.refs.risk = TRUE)
names(res_snps_dat)
@
These results allow us to create a useful plot showing the distribution of risks in our reference dataset 
and to add the risks of the three women to see where they fall on the population distribution.
<<plot-1b, fig=TRUE>>=
plot(density(res_snps_dat$refs.risk),
           xlim = c(0.04,0.18), xlab = "Absolute Risk of Breast Cancer",
           main = "Referent SNP-only Risk Distribution: Ages 50-80 years")
abline(v = res_snps_dat$risk, col = "red")
legend("topright", legend = "New profiles", col = "red", lwd = 1)
@

\section*{Example 2: Breast cancer risk model with risk-factors and SNPs}

In this example, we will estimate the risk of breast cancer in ages 50-80 by fitting a model with classical risk factors and 72 SNPs, 
with three specific covariate profiles supplied for estimation (with some missing data). 
More details on risk factors are available in the manual.

<<ex-2>>=
res_covs_snps = computeAbsoluteRisk(model.formula = bc_model_formula,
                                model.cov.info = bc_model_cov_info,
                                model.snp.info = bc_72_snps,
                                model.log.RR = bc_model_log_or,
                                model.ref.dataset = ref_cov_dat,
                                model.disease.incidence.rates = bc_inc,
                                model.competing.incidence.rates = mort_inc,
                                model.bin.fh.name = "famhist",
                                apply.age.start = 50,
                                apply.age.interval.length = 30,
                                apply.cov.profile = new_cov_prof,
                                apply.snp.profile = new_snp_prof,
                                return.refs.risk = TRUE)
@
In addition to summarizing and plotting the risk estimates, iCARE includes an option to view more detailed output, by calling:
<<fit-2>>=
print(res_covs_snps$details)
@
\section*{Illustration of the validation component}
We want to validate a model for predicting absolute risk of disease based on a combined model of classical risk 
 factors and 72 SNPs using the nested case-control dataset. 

The first step is to compute sampling weights. We fit a logistic regression model of inclusion depending on the case/control status, 
 age of study entry and observed followup using the R function \textbf{glm}, as shown below:
<<sampling-weights>>=
validation.cohort.data$inclusion = 0
subjects_included = intersect(validation.cohort.data$id,
                            validation.nested.case.control.data$id)
validation.cohort.data$inclusion[subjects_included] = 1

validation.cohort.data$observed.followup =
                        validation.cohort.data$study.exit.age -
                            validation.cohort.data$study.entry.age

selection.model = glm(inclusion ~ observed.outcome
                       * (study.entry.age + observed.followup),
                              data = validation.cohort.data,
                              family = binomial(link = "logit"))
validation.nested.case.control.data$sampling.weights =
        selection.model$fitted.values[validation.cohort.data$inclusion == 1]
@
The next step is to call the \textbf{ModelValidation} function to implement the validation analysis.
<<model-validation>>=
data = validation.nested.case.control.data
risk.model = list(model.formula = bc_model_formula,
                   model.cov.info = bc_model_cov_info,
                   model.snp.info = bc_72_snps,
                   model.log.RR = bc_model_log_or,
                   model.ref.dataset = ref_cov_dat,
                   model.ref.dataset.weights = NULL,
                   model.disease.incidence.rates = bc_inc,
                   model.competing.incidence.rates = mort_inc,
                   model.bin.fh.name = "famhist",
                   apply.cov.profile = data[,all.vars(bc_model_formula)[-1]],
                   apply.snp.profile = data[,bc_72_snps$snp.name],
                   n.imp = 5, use.c.code = 1, return.lp = TRUE,
                   return.refs.risk = TRUE)

output = ModelValidation(study.data = data,
                         total.followup.validation = TRUE,
                         predicted.risk.interval = NULL,
                         iCARE.model.object = risk.model,
                         number.of.percentiles = 10)
@
\setkeys{Gin}{width=1.3\textwidth}
We can also produce a set of useful plots showing the results of the validation analysis.
<<plot-model-validation, fig=TRUE, height = 12, width = 9>>=
plotModelValidation(study.data = data,validation.results = output)
@
\section*{Session Information}
<<sessionInfo>>=
sessionInfo()
@ 

\end{document}

%\VignetteIndexEntry{iCARE Vignette}
%\VignettePackage{iCARE}
%\VigetteDepends{iCARE}

\documentclass[a4paper]{article}
\begin{document}

\title{iCARE(Individualized Coherent Absolute Risk Estimators) Package}
\maketitle

<<start>>=
library(iCARE)
@

\section*{Example 1.A}

Load the breast cancer data.
<<load-data>>=
data("bc_data", package="iCARE")
@

In this example, we will estimate the risk of breast cancer in ages 50-80.
A SNP-only model is fit, with no specific genotypes supplied for estimation.
The population disease rates are from SEER.
<<model-1a>>=
res_snps_miss = computeAbsoluteRisk(model.snp.info = bc_72_snps, 
                      model.disease.incidence.rates = bc_inc,
                      model.competing.incidence.rates = mort_inc, 
                      apply.age.start = 50, 
                      apply.age.interval.length = 30,
                      return.refs.risk=TRUE)
@

Compute a summary of the risks and visualize the results
<<summary-1a, fig=TRUE>>=
summary(res_snps_miss$risk)
summary(res_snps_miss$refs.risk)
plot(density(res_snps_miss$risk), lwd=2, 
  main="SNP-only Risk Stratification: Ages 50-80", 
  xlab="Absolute Risk of Breast Cancer")
@

\section*{Example 1.B}

In this example, we will again estimate the risk of breast cancer in ages 50-80.
This time however, three specific genotypes are supplied for estimation (with some missing data).
The argument return.refs.risk = TRUE, includes the referent dataset risks be included in results.
<<model 1b>>=
res_snps_dat = computeAbsoluteRisk(model.snp.info = bc_72_snps, 
                                model.disease.incidence.rates = bc_inc,
                                model.competing.incidence.rates = mort_inc, 
                                apply.age.start = 50, 
                                apply.age.interval.length = 30, 
                                apply.snp.profile = new_snp_prof, 
                                return.refs.risk = TRUE)
names(res_snps_dat)
@

Visualize the Results
<<plot-1b, fig=TRUE>>=
plot(density(res_snps_dat$refs.risk), lwd=2, 
   main="Referent SNP-only Risk Distribution: Ages 50-80",
   xlab="Absolute Risk of Breast Cancer")
abline(v=res_snps_dat$risk, col="red")
legend("topright", legend="New Profiles", col="red", lwd=1)
@

\section*{Example 2}

In this example, we will estimate the risk of breast cancer in ages 50-80 by
fitting a model with 13 risk factors and 72 SNPs.


<<fit-2>>=
res_covs_snps = computeAbsoluteRisk(model.formula=bc_model_formula, 
                                         model.cov.info=bc_model_cov_info,
                                         model.snp.info=bc_72_snps,
                                         model.log.RR=bc_model_log_or,
                                         model.ref.dataset=ref_cov_dat,
                                         model.disease.incidence.rates=bc_inc,
                                         model.competing.incidence.rates=mort_inc, 
                                         model.bin.fh.name="famhist",
                                         apply.age.start=50, 
                                         apply.age.interval.length=30,
                                         apply.cov.profile=new_cov_prof,
                                         apply.snp.profile=new_snp_prof, 
                                         return.refs.risk=TRUE)
@

Display details of the fit
<<fit-2>>=
print(res_covs_snps$details)
@

\section*{Session Information}
<<sessionInfo>>=
sessionInfo()
@ 

\end{document}

% Generated by roxygen2 (4.1.1): do not edit by hand
% Please edit documentation in R/absolute_risk_codev.R
\name{computeAbsoluteRisk}
\alias{computeAbsoluteRisk}
\title{Building and Applying an Absolute Risk Model}
\usage{
computeAbsoluteRisk(model.formula = NULL, model.cov.info = NULL,
  model.snp.info = NULL, model.log.RR = NULL, model.ref.dataset = NULL,
  model.ref.dataset.weights = NULL, model.disease.incidence.rates,
  model.competing.incidence.rates = NULL, model.bin.fh.name = NA,
  n.imp = 5, apply.age.start, apply.age.interval.length,
  apply.cov.profile = NULL, apply.snp.profile = NULL, use.c.code = 1,
  return.lp = FALSE, return.refs.risk = FALSE)
}
\arguments{
\item{model.formula}{an object of class \code{formula}: a symbolic description of the model to be fitted, e.g. Y~Parity+FamilyHistory.}

\item{model.cov.info}{contains information about the risk factors in the model ;  
   a main list containing a list for each covariate, which must have the fields:\cr
\itemize{ \item \code{"name"} : a string with the covariate name, matching name in model.formula \cr
\item \code{"type"} : a string that is either "continuous" or "factor".}
If factor variable, then:\cr
\itemize{\item \code{"levels"} : vector with strings of level names  \cr
\item \code{"ref"} : optional field, string with name of referent level
}}

\item{model.snp.info}{dataframe with three columns, named: [ "snp.name", "snp.odds.ratio", "snp.freq" ]}

\item{model.log.RR}{vector with log odds ratios corresponding to the model params; no intercept; 
      names must match design matrix arising from \code{model.formula} and \code{model.cov.info}; 
      check names using function \code{check_design_matrix()}.}

\item{model.ref.dataset}{dataframe of risk factors for a sample of subjects representative of underlying population, no missing values. 
    Variables must be in same order with same names as in \code{model.formula}.}

\item{model.ref.dataset.weights}{optional vector of sampling weights for \code{model.ref.dataset}.}

\item{model.disease.incidence.rates}{two column matrix [ integer ages, incidence rates] or three column matrix [start age, end age, rate] 
    with incidence rate of disease. Must fully cover age interval for estimation.}

\item{model.competing.incidence.rates}{two column matrix [ integer ages, incidence rates] or three column matrix [start age, end age, rate] 
  with incidence rate of competing events. Must fully cover age interval for estimation.}

\item{model.bin.fh.name}{string name of family history variable, if in model. This must refer to a variable that only takes values 0,1, NA.}

\item{n.imp}{integer value for number of imputations for handling missing SNPs.}

\item{apply.age.start}{single integer or vector of integer ages for the start of the interval over which to compute absolute risk.}

\item{apply.age.interval.length}{single integer or vector of integer years over which absolute risk should be computed.}

\item{apply.cov.profile}{dataframe containing the covariate profiles for which absolute risk will be computed. Covariates must be in same order 
   with same names as in \code{model.formula}.}

\item{apply.snp.profile}{data frame with observed SNP data (coded 0,1, 2, or NA). May have missing values.}

\item{use.c.code}{binary indicator of whether to run the c program for fast computation.}

\item{return.lp}{binary indicator of whether to return the linear predictor for each subject in apply.cov.profile.}

\item{return.refs.risk}{binary indicator of whether to return the absolute risk prediction for each subject in \code{model.ref.dataset}.}
}
\value{
This function returns a list of results objects, including: \cr
\itemize{ \item \code{risk} : absolute risk estimates over the specified interval for subjects given by \code{apply.cov.profile} \cr
\item \code{details}: dataframe with the start of the interval, the end of the interval, the covariate profile, and the risk estimates for each individual \cr
\item \code{beta.used} : the log odds ratios used in the model \cr
\item \code{lps} : linear predictors for subjects in \code{model.cov.profile}, if requested by \code{return.lp} \cr
\item \code{refs.risk} : absolute risk estimates for subjects in \code{model.ref.dataset}, if requested by \code{return.refs.risk}; 
       computes for first age interval provided  }
}
\description{
This function is used to build absolute risk models and apply them to estimate absolute risks.
}
\details{
Individualized Coherent Absolute Risk Estimators (iCARE) is a tool that allows researchers to quickly build models for absolute risk and 
apply them to estimate individuals' risk based on a set of user defined input parameters. The software gives users the flexibility to change 
or update models rapidly based on new risk factors or tailor models to different populations based on the specification of simply three input arguments: 
  \itemize{ 
    \item (1) a model for relative risk assumed to be externally derived 
    \item (2) an age-specific disease incidence rate and 
    \item (3) the distribution of risk factors for the population of interest.
  } The tool can handle missing information on risk factors for risk estimation using an approach where all estimates are derived 
   from a single model through appropriate model averaging.
}
\examples{
data(bc_data, package="iCARE")
results = computeAbsoluteRisk(model.formula=bc_model_formula, 
                                 model.cov.info    = bc_model_cov_info,
                                 model.snp.info    = bc_72_snps,
                                 model.log.RR      = bc_model_log_or,
                                 model.ref.dataset = ref_cov_dat,
                                 model.disease.incidence.rates   = bc_inc,
                                 model.competing.incidence.rates = mort_inc, 
                                 model.bin.fh.name = "famhist",
                                 apply.age.start    = 50, 
                                 apply.age.interval.length = 30,
                                 apply.cov.profile  = new_cov_prof,
                                 apply.snp.profile  = new_snp_prof, 
                                 return.refs.risk   = TRUE)
summary(results)
plot(results, main="Risk")
boxplot(results$risk ~ new_cov_prof$famhist, na.rm=TRUE)
}

% --- Source file: iCARE.Rd ---
\name{iCARE}
\alias{iCARE}
\docType{package}
\title{
A Tool for Individualized Coherent Absolute Risk Estimation (iCARE)
}
\description{
Individualized Coherent Absolute Risk Estimators (iCARE) is a tool that allows researchers to quickly 
build models for absolute risk and apply them to estimate individuals' risk based on a set of user defined 
input parameters. The software gives users the flexibility to change or update models rapidly based on new 
risk factors or tailor models to different populations based on the specification of simply three input arguments: 
(1) a model for relative risk assumed to be externally derived 
(2) an age-specific disease incidence rate and 
(3) the distribution of risk factors for the population of interest. 
The tool can handle missing information on risk factors for risk estimation using an approach where all estimates 
are derived from a single model through appropriate model averaging.
}
\details{
The main functions for building and applying an absolute risk model are \code{\link{computeAbsoluteRisk}} 
and \code{\link{computeAbsoluteRiskSplitInterval}}.  
The first of these computes absolute risks over the specified time interval using a single set of paramters.  
The second provides more advanced functionality and computes absolute risk over the interval in two parts.  

\code{\link{computeAbsoluteRiskSplitInterval}} allows the user compute absolute risk over the interval in two parts,
 incorporating two different sets of paramters before and after a specified cutpoint.  
This function allows a different cutpoint for each covariate profile if desired. 
The function for validating an absolute risk model is \code{\link{ModelValidation}}, and
   \code{\link{plotModelValidation}} can be called for producing plots 
   for model calibration, model discrimination and incidence rates.
}
%\references{ }
\author{Paige Maas, Parichoy Pal Choudhury, Nilanjan Chatterjee and William Wheeler <wheelerb@imsweb.com>}
\keyword{package}

\name{plotModelValidation}
\alias{plotModelValidation}
\alias{plot}
\title{Model Validation Plot}
\usage{
plotModelValidation(study.data, validation.results,
                    dataset = "Example Dataset",
                    model.name = "Example Model",
                    x.lim.absrisk = "",
                    y.lim.absrisk = "", 
                    x.lab.absrisk = "Expected Absolute Risk (\%)", 
                    y.lab.absrisk = "Observed Absolute Risk (\%)", 
                    x.lim.RR = "",
                    y.lim.RR = "", x.lab.RR = "Expected Relative Risk", 
                    y.lab.RR = "Observed Relative Risk",
                    risk.score.plot.kernel = "gaussian",
                    risk.score.plot.bandwidth = "nrd0",
                    risk.score.plot.percent.smooth = 50)
}
\arguments{
  \item{study.data}{See \code{\link{ModelValidation}} }
  \item{validation.results}{List returned from \code{\link{ModelValidation}} }
  \item{dataset}{Name and type of dataset to be displayed in the output, 
           e.g., "PLCO Full Cohort" or "Full Cohort Simulation"}
  \item{model.name}{Name of the model to be displayed in output, 
                     e.g., "Synthetic Model" or "Simulation Setting"}
  \item{x.lim.absrisk}{ Vector of length two specifying the x-axes limits in the
                               absolute risk calibration plot. If not specified, then default
                               limits will be computed.}
  \item{y.lim.absrisk}{ Vector of length two specifying the y-axes limits in the
                               absolute risk calibration plot. If not specified, then default
                               limits will be computed.}
  \item{x.lab.absrisk}{ String specifying the x-axes label in the  
                               absolute risk calibration plot.
                               The default is "Expected Absolute Risk (\%)".}
  \item{y.lab.absrisk}{ String specifying the y-axes label in the  
                               absolute risk calibration plot.
                               The default is "Observed Absolute Risk (\%)."}
  \item{x.lim.RR}{ Vector of length two specifying the x-axes limits in the
                               relative risk calibration plot. If not specified, then default
                               limits will be computed.}
  \item{y.lim.RR}{ Vector of length two specifying the y-axes limits in the
                               relative risk calibration plot. If not specified, then default
                               limits will be computed.}
  \item{x.lab.RR}{ String specifying the x-axes label in the  
                               relative risk calibration plot.
                               The default is "Expected Relative Risk".}
  \item{y.lab.RR}{ String specifying the y-axes label in the  
                               relative risk calibration plot.
                               The default is "Observed Relative Risk".}
  \item{risk.score.plot.kernel}{Character string giving the smoothing kernel to be used by the density function used 
         internally to plot the density of the risk scores. It should be one of "gaussian", "rectangular", "triangular",
          "epanechnikov", "biweight", "cosine" or "optcosine", with default "gaussian".}
  \item{risk.score.plot.bandwidth}{The options for bandwidth selection used by the density function internally to plot 
        the density of the risk scores. The options are one of the following: "nrd0", "nrd", "ucv", "bcv", "SJ-ste", "SJ-dpi"
        with the default being "nrd0". More information on these different options is available in the help pages that can be 
       accessed from R using the command \code{?bw.nrd}.}
  \item{risk.score.plot.percent.smooth}{Percentage of the number of sample points used for determining the number of equally 
      spaced points at which the density of the risk score is to be estimated. This number supplies the input parameter "n" to the 
      density function used internally to plot the densities of the risk score. The default value is 50.}  
}

\value{
This function returns NULL \cr
}
\description{
This function is used to create plots for model calibration, model discrimination and incidence rates.
}
%\details{}
\seealso{\code{\link{ModelValidation}}}
\examples{
data(bc_data, package="iCARE")
validation.cohort.data$inclusion = 0
subjects_included = intersect(validation.cohort.data$id, 
                              validation.nested.case.control.data$id)
validation.cohort.data$inclusion[subjects_included] = 1

validation.cohort.data$observed.followup = validation.cohort.data$study.exit.age - 
  validation.cohort.data$study.entry.age

selection.model = glm(inclusion ~ observed.outcome 
                      * (study.entry.age + observed.followup), 
                      data = validation.cohort.data, 
                      family = binomial(link = "logit"))

validation.nested.case.control.data$sampling.weights =
  selection.model$fitted.values[validation.cohort.data$inclusion == 1]

set.seed(50)

data = validation.nested.case.control.data

snpDat     = bc_72_snps
form       = diagnosis ~ famhist + as.factor(parity)
info       = list(bc_model_cov_info[[1]], bc_model_cov_info[[3]])
vars       = all.vars(form)[-1]
risk.model = list(model.formula = form,
                  model.cov.info = info,
                  model.snp.info = snpDat,
                  model.log.RR = bc_model_log_or[c(1, 8:11)],
                  model.ref.dataset = ref_cov_dat[, vars],
                  model.ref.dataset.weights = NULL,
                  model.disease.incidence.rates = bc_inc,
                  model.competing.incidence.rates = mort_inc,
                  model.bin.fh.name = "famhist",
                  apply.cov.profile = data[,vars],
                  apply.snp.profile = data[,snpDat$snp.name],
                  n.imp = 5, use.c.code = 1, return.lp = TRUE,
                  return.refs.risk = TRUE)

# Not run since it can take a few minutes
#output = ModelValidation(study.data = data, total.followup.validation = TRUE,
#          predicted.risk.interval = NULL, iCARE.model.object = risk.model,
#          number.of.percentiles = 10)

plot(output)

}

\name{ModelValidation}
\alias{ModelValidation}
\title{Model Validation}
\usage{
ModelValidation(study.data, 
                total.followup.validation = FALSE,
                predicted.risk = NULL, 
                predicted.risk.interval = NULL, 
                linear.predictor = NULL, 
                iCARE.model.object = 
                  list(model.formula = NULL,
                       model.cov.info = NULL,
                       model.snp.info = NULL,
                       model.log.RR = NULL,
                       model.ref.dataset = NULL,
                       model.ref.dataset.weights = NULL,
                       model.disease.incidence.rates = NULL,
                       model.competing.incidence.rates = NULL,
                       model.bin.fh.name = NA,
                       apply.cov.profile  = NULL,
                       apply.snp.profile = NULL, 
                       n.imp = 5, use.c.code = 1,
                       return.lp = TRUE, 
                       return.refs.risk = TRUE),
                number.of.percentiles = 10,
                reference.entry.age = NULL, 
                reference.exit.age = NULL,
                predicted.risk.ref = NULL,
                linear.predictor.ref = NULL,
                linear.predictor.cutoffs = NULL,
                dataset = "Example Dataset", 
                model.name = "Example Risk Prediction Model")            
}
\arguments{
  \item{study.data}{Data frame which includes the variables below.
    \itemize{
      \item \code{observed.outcome}: 1 if disease has occurred by the end of followup, 0 if censored
      \item \code{study.entry.age}: age (in years) of entering the cohort
      \item \code{study.exit.age}: age (in years) of last followup visit
      \item \code{time.of.onset}: time (in years) of onset of disease; note that all subjects are disease free at 
          the time of entry and for those who do not develop disease by end of followup it is Inf
      \item \code{sampling.weights}: for a case-control study nested within a cohort study, this is a vector
          of sampling weights for each subject, i.e., probability of inclusion into the sample
    }
  }
  \item{total.followup.validation}{logical; TRUE if risk validation is performed over the total followup, 
      for all other cases (e.g., 5 year or 10 year risk validation) it is FALSE}
  \item{predicted.risk}{vector of predicted risks; should be supplied if risk prediction is done by some 
       method other than that implemented in \code{iCARE}; default is NULL}
  \item{predicted.risk.interval}{scalar or vector denoting the number of years after entering 
       the study over which risk validation is desired (e.g., 5 for validating a model for 5 year risk) 
       if \code{total.followup.validation = FALSE}; 
        if \code{total.followup.validation = TRUE},
        it can be set to NULL}
  \item{linear.predictor}{vector of risk scores for each subject, i.e. \code{x*beta}, where \code{x} is the 
       vector of risk factors and \code{beta} is the vector of log relative risks; in the current version if 
       both the arguments \code{predicted.risk} and \code{linear.predictor} are supplied the function will use the 
       supplied estimates to perform model validation, otherwise the function will compute these estimates
       using the \code{computeAbsoluteRisk} function}
  \item{iCARE.model.object}{A named list containing the input arguments to the function \code{\link{computeAbsoluteRisk}}.
                   The names in this list must match the argument names.
                   See \code{\link{computeAbsoluteRisk}}
                   }
  \item{number.of.percentiles}{the number of percentiles of the risk score that determines the number 
      of strata over which the risk prediction model is to be validated, default = 10}
  \item{reference.entry.age}{age of entry to be specified for computing absolute risk of the reference population}
  \item{reference.exit.age}{age of exit to be specified for computing absolute risk of the reference population}
  \item{predicted.risk.ref}{predicted absolute risk in the reference population assuming the entry age to be 
       as specified in \code{reference.entry.age} and exit age to be as specified in \code{reference.exit.age}}
  \item{linear.predictor.ref}{vector of risk scores for the reference population}
  \item{linear.predictor.cutoffs}{user specified cut-points for the linear predictor to define categories for
       absolute risk calibration and relative risk calibration}
  \item{dataset}{name and type of dataset to be displayed in the output, e.g.,
        "PLCO Full Cohort" or "Full Cohort Simulation"}
  \item{model.name}{name of the model to be displayed in output, e.g., "Synthetic Model" or "Simulation Setting"}
}

\value{
This function returns a list of the following objects: \cr
  \itemize{
    \item \code{Subject_Specific_Observed_Outcome}: observed outcome after adjusting the observed  followup according 
        to the risk prediction interval: 1 if disease has occurred by the end of followup, 0 if censored 
    \item \code{Risk_Prediction_Interval}: Character object showing the interval of risk prediction 
              (e.g., 5 years). If the risk prediction is over the total followup of the study,
              this reads "Observed Followup" 
    \item \code{Adjusted.Followup}: followup time (in years) after adjusting the observed followup according to the risk 
           prediction interval 
    \item \code{Subject_Specific_Predicted_Absolute_Risk}: predicted absolute risk of disease for each subject 
    \item \code{Reference_Absolute_Risk}: predicted absolute risk in the reference population 
    \item \code{Subject_Specific_Risk_Score}: estimated risk score for each subject; the missing covariates are handled 
                internally using the imputation in \code{iCARE} 
    \item \code{Reference_Risk_Score}: risk score for the reference population 
    \item \code{Population_Incidence_Rate}: age specific disease incidence rate in the population 
    \item \code{Study_Incidence_Rate}: estimated age specific incidence rate in the study 
    \item \code{Category_Results}: observed and predicted absolute risks and observed and predicted relative risks 
          in each category defined by the risk score 
    \item \code{Category_Specific_Observed_Absolute_Risk}: Observed absolute risk in each category defined by the risk score 
    \item \code{Category_Specific_Predicted_Absolute_Risk}: Predicted absolute risk in each category defined by the risk score 
    \item \code{Category_Specific_Observed_Relative_Risk}: Observed relative risk in each category defined by the risk score 
    \item \code{Category_Specific_Predicted_Relative_Risk}: Predicted relative risk in each category defined by the risk score 
    \item \code{Variance_Matrix_Absolute_Risk}: Variance-covariance matrix of the vector of cateogry specific absolute risks 
    \item \code{Variance_Matrix_LogRelative_Risk}: Variance-covariance matrix of the vector of cateogry specific relative risks 
    \item \code{Hosmer_Lemeshow_Results}: results of the Hosmer-Lemeshow type chisquare test comparing the observed and predicted absolute risks 
    \item \code{HL_pvalue}: pvalue of the Hosmer-Lemeshow type chisquare test 
    \item \code{RR_test_result}: results of the chisquare test comparing the observed and predicted relative risks 
    \item \code{RR_test_pvalue}: pvalue of the chisquare test of relative risk 
    \item \code{AUC}: estimate of the Area Under the Curve (AUC) defined as the probability that for a randomly sampled 
            case-control pair the case has a higher risk score than the control; for the full cohort setting we compute 
            the empirical proportion and for the nested case-control setting we compute the inverse probability weighted estimator 
    \item \code{Variance_AUC}: estimate of the variance of Area Under the Curve (AUC): for the full cohort setting the regular
        asymptotic variance is estimated and for the nested case-control setting the influence function based variance 
        estimate of the inverse probability weighted variance estimator is computed 
    \item \code{CI_AUC}: 95 percent Wald based confidence interval of Area Under the Curve (AUC) using the asymptotic variance 
    \item \code{Overall_Expected_to_Observed_Ratio}: The overall ratio of the expected risk to the observed risk 
    \item \code{CI_Overall_Expected_to_Observed_Ratio}: 95 percent Wald based confidence interval of the overall ratio of the expected risk to the observed risk 
  }
}
\description{
This function is used to validate absolute risk models.
}
%\details{}
\seealso{\code{\link{computeAbsoluteRisk}}}
\examples{

data(bc_data, package="iCARE")
validation.cohort.data$inclusion = 0
subjects_included = intersect(validation.cohort.data$id, 
                              validation.nested.case.control.data$id)
validation.cohort.data$inclusion[subjects_included] = 1

validation.cohort.data$observed.followup = validation.cohort.data$study.exit.age - 
  validation.cohort.data$study.entry.age

selection.model = glm(inclusion ~ observed.outcome 
                      * (study.entry.age + observed.followup), 
                      data = validation.cohort.data, 
                      family = binomial(link = "logit"))

validation.nested.case.control.data$sampling.weights =
  selection.model$fitted.values[validation.cohort.data$inclusion == 1]

set.seed(50)

data = validation.nested.case.control.data

snpDat     = bc_72_snps
form       = diagnosis ~ famhist + as.factor(parity)
info       = list(bc_model_cov_info[[1]], bc_model_cov_info[[3]])
vars       = all.vars(form)[-1]
risk.model = list(model.formula = form,
                  model.cov.info = info,
                  model.snp.info = snpDat,
                  model.log.RR = bc_model_log_or[c(1, 8:11)],
                  model.ref.dataset = ref_cov_dat[, vars],
                  model.ref.dataset.weights = NULL,
                  model.disease.incidence.rates = bc_inc,
                  model.competing.incidence.rates = mort_inc,
                  model.bin.fh.name = "famhist",
                  apply.cov.profile = data[,vars],
                  apply.snp.profile = data[,snpDat$snp.name],
                  n.imp = 5, use.c.code = 1, return.lp = TRUE,
                  return.refs.risk = TRUE)

# Not run since it can take a few minutes
# output = ModelValidation(study.data = data, total.followup.validation = TRUE,
#      predicted.risk.interval = NULL, iCARE.model.object = risk.model,
#      number.of.percentiles = 10)
output

}

% --- Source file: bc_data.Rd ---
\name{bc_data}
\alias{bc_data}
\alias{bc_72_snps}
\alias{bc_inc}
\alias{bc_model_formula}
\alias{bc_model_cov_info}
\alias{bc_model_log_or}
\alias{bc_model_log_or_post_50}
\alias{mort_inc}
\alias{new_cov_prof}
\alias{ref_cov_dat}
\alias{new_snp_prof}
\alias{ref_cov_dat_post_50}
\alias{validation.nested.case.control.data}
\alias{validation.cohort.data}
\alias{output}


%\docType{package}
\title{
  Data for examples
}
\description{
 Example data for \code{\link{computeAbsoluteRisk}}, \code{\link{computeAbsoluteRiskSplitInterval}}, 

 \code{\link{ModelValidation}}, and \code{\link{plotModelValidation}}.
}
\details{ 
\itemize{
\item \code{bc_model_cov_info}: a main list containing information on family history, age at menarche (years), 
   parity, age at first birth (years), age at menopause (years), height (meters), Body Mass Index (kg/sq.m.),
  use of hormone replacement therapy, use of estrogen and progesterone combined therapy, 
  use of estrogen only therapy, current use of hormone replacement therapy, alcohol (drinks/week), smoking status.; 
   information on each risk factor is given as a list 
\item \code{bc_model_formula}: formula for the specification of the models with risk factors
\item \code{bc_72_snps}: contains published SNP information from reference:
  Michailidou K, Lindstrom S, Dennis J, Beesley J, Hui S, Kar S, Lemacon A, Soucy P, Glubb D, Rostamianfar A, 
 et al. (2017) Association analysis identifies 65 new breast cancer risk loci. Nature 551:92-94
\item \code{bc_model_log_or}: vector of log-odds ratios of family history, age at menarche (years), parity, 
age at first birth (years), age at menopause (years), height (meters), Body Mass Index (kg/sq.m.), 
use of hormone replacement therapy, use of estrogen and progesterone combined therapy, 
use of estrogen only therapy, current use of hormone replacement therapy, alcohol (drinks/week), smoking status. 
\item \code{bc_model_log_or_post_50}: vector of log-odds ratios of family history, age at menarche (years), parity, age at first birth (years), 
age at menopause (years), height (meters), Body Mass Index (kg/sq.m.), use of hormone replacement therapy, 
use of estrogen and progesterone combined therapy, use of estrogen only therapy,
 current use of hormone replacement therapy, alcohol (drinks/week), smoking status. for women 50 years or older 
\item \code{ref_cov_dat}: contains individual level reference dataset of risk factors representative of the underlying population imputed using reference (4) and (5) 
\item \code{ref_cov_dat_post_50}: contains individual level reference dataset of the risk factors for women aged 50 years or older 
\item \code{bc_inc}: contains age-specific incidence rates of breast cancer from reference (3) 
\item \code{mort_inc}: contains age-specific incidence rates of all-cause mortality from reference (1) below 
\item \code{new_cov_prof}: Information on family history, age at menarche (years), parity, age at first birth (years), age at menopause (years), height (meters), 
  Body Mass Index (kg/sq.m.), use of hormone replacement therapy, use of estrogen and progesterone combined therapy, 
  use of estrogen only therapy, current use of hormone replacement therapy, alcohol (drinks/week), 
  smoking status for three women (given for illustration of absolute risk prediction) 
\item \code{new_snp_prof}: Information on 72 breast cancer associated SNPs for three women (given for illustration of absolute risk prediction) 
\item \code{validation.cohort.data}: Simulated  full cohort dataset of 50,000 women for illustration 
of model validation. The variables are: 
 \itemize{
  \item \code{id}: Subject id 
  \item \code{famhist}: Family history; binary indicator of presence/absence of disease among first degree relatives 
  \item \code{parity}: number of child births categorized as nulliparous (ref), 1 births, 2 births, 3 births, 4+ births
  \item \code{menarche_dec}: categories of age at menarche (years) with levels: less than 11,11-11.5,11.5-12,12-13(ref),13-14,14-15, greater than 15
  \item \code{birth_dec}: categories of age at first birth (years) with levels: less than 19 (ref), 19-22,22-23,23-25,25-27,27-30,30-34,34-38, greater than 38
  \item \code{agemeno_dec}: categories of age at menopause (years) with levels: less than 40 (ref), 40-45, 45-47, 47-48, 48-50, 50-51, 51-52, 52-53, 53-55, greater than 55
  \item \code{height_dec}: categories of height (meters) with levels: less than 1.55, 1.55-1.57, 1.57-1.60, 1.60-1.61, 1.61-1.63, 1.63-1.65, 1.65-1.66, 1.66-1.68, 1.68-1.71
  \item \code{bmi_dec}: categories of body mass index (kg/sq.m.) with levels: less than 21.5 (ref), 21.5-23, 23-24.2, 24.2-25.3, 25.3-26.5, 26.5-27.8, 27.8-29.3, 29.3-31.4, 31.4-34.6
  \item \code{rd_menohrt}: use of hormone replacement therapy with levels: premenopausal (ref), postmenopausal and never HRT user, postmenopausal and ever HRT user
  \item \code{rd2_everhrt_c}: binary indicator for postmenopausal and ever user of estrogen and progesterone combined therapy
  \item \code{rd2_everhrt_e}: binary indicator for postmenopausal and ever user of estrogen only therapy
  \item \code{rd2_currhrt}: binary indicator of postmenopausal and current HRT user
  \item \code{alcoholdweek_dec}: alcohol in drinks per week categorized into levels: none (ref), 0-0.4, 0.4-0.8, 0.8-1.5, 1.5-3.2, 3.2-5.7, 5.7-9.8, >9.8
  \item \code{ever_smoke}: binary indicator for ever smoker 
  \item \code{study.entry.age}: age of study entry 
  \item \code{study.exit.age}: age of study exit 
  \item \code{observed.outcome}: binary indicator of disease status (yes/no) 
  \item \code{time.of.onset}: time (in years) since study entry to the development of disease; 
   for subjects who have not developed disease beyond the observed followup, it is set to Inf 
  \item \code{observed.followup}: number of years the subject is followed up in the study 
   (difference between the age of study exit and age of study entry)
 } % END: itemize
\item \code{validation.nested.case.control.data}: A simulated example of a case-control study of 5285 women, 
  nested within the full cohort.
 In addition to the variables given above, it has information on the 72 breast cancer associated SNPs with 
variable names being the rs-identifiers.
\item \code{output}: object returned from \code{\link{computeAbsoluteRisk}}
} % END itemize
} % END: details
\examples{

 temp <- data(bc_data, package="iCARE")

 # Display the object names
 temp

}
\keyword{data}

\references{
(1)  Centers for Disease Control and Prevention (CDC), National Center for Health Statistics (NCHS). Underlying Cause of Death 1999-2011 on CDC WONDER Online Database, released 2014. Data are from the Multiple Cause of Death Files, 1999-2011, as compiled from data provided by the 57 vital statistics jurisdictions through the Vital Statistics Cooperative Program. \cr
 Accessed at http://wonder.cdc.gov/ucd-icd10.html on Aug 26, 2014.

(2)  Michailidou K, Beesley J, Lindstrom S, et al. Genome-wide association analysis of more than 120,000 individuals identifies 15 new susceptibility loci for breast cancer. Nature genetics 2015;47:373-80.

(3)  Surveillance, Epidemiology, and End Results (SEER) Program SEER*Stat Database: Incidence - SEER 18 Regs Research Data, Nov 2011 Sub, Vintage 2009 Pops (2000-2009) <Katrina/Rita Population Adjustment> - Linked To County Attributes - Total U.S., 1969-2010 Counties. In: National Cancer Institute D, Surveillance Research Program, Surveillance Systems Branch, ed. SEER18 ed.

(4)  2010 National Health Interview Survey (NHIS) Public Use Data Release, NHIS Survey Description. 2011. \cr
(Accessed at ftp://ftp.cdc.gov/pub/Health_Statistics/NCHS/Dataset_Documentation/NHIS/2010/srvydesc.pdf.)

(5)  Centers for Disease Control and Prevention (CDC). National Center for Health Statistics (NCHS). National Health and Nutrition Examination Survey Questionnaire. Hyattsville, MD: U.S. Department of Health and Human Services, Centers for Disease Control and Prevention; 2010.

}
